/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreeModel");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.NodeTypes");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.navigator.AbstractModel");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.client.internal.option");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

(function() {

var NodeTypes = com.ibm.asq.common.web.ui.internal.widgets.navigator.NodeTypes;
var CategoryClient  = com.ibm.rqm.planning.web.client.internal.CategoryClient;
var ServiceResponseHandler		    = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var getCurrentProjectArea 			= jazz.app.context.get;
var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;

var CategoryValueIcon =  "category-value-tree-icon-image";
var CategorySubTypeIcon =  "category-tree-icon-image";
var CategoryRootTypeIcon = "category-folder-tree-icon-image";
var DisabledStyle = " disabled-category-style";


var nameComparator = function(object1, object2) {	
    var name1 = object1.name ? object1.name.toLowerCase() : object1.name;
    var name2 = object2.name ? object2.name.toLowerCase() : object2.name;
    return (name1 > name2) ? 1 : (name1 == name2) ? 0 : -1;
};

var sortArrayByName = function(array) {
	if (!dojo.isArray(array)) {
		// If by any reason we don't receive an array, just return
		// whatever we received.
		return array;
	} else {
		return (array || []).sort(nameComparator);
	}
};


dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryModelDataProvider",
		null , {
	
	getCategoryOrder: function(onComplete, categoryTypes){		 		
		var option = new com.ibm.asq.common.web.client.internal.option();
		var sortCategoryTypes = com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreeModel.sortCategoryTypes;
		var self = this;     	
		var handler =  {
				success: function(results) {
					self.loaded = true;	
					var orderPrefEntry = results && results.value ? dojo.fromJson(results.value) : {};
					orderPrefEntry.itemId = results?results.itemId:null;
					self.categoryTypes =  Utility.sortCategoryTypes(orderPrefEntry, categoryTypes);
					self.orderPrefEntry = orderPrefEntry;
					onComplete(self.categoryTypes);
				}, 
				error: function(error){
					self.loaded = true;	

					console.error("Could not get custom attributes order ", error);
				}
		};
		var context = Utility.getCategoryOrderContext(this.artifactType);
		var responseHandler = new ServiceResponseHandler(handler, "success", "error");
		option.getProjectPreferences( responseHandler,  getCurrentProjectArea().itemId, context);
	}, 		
     
	/**
	 * Service calls
	 */
	categoryTypesProvider: function(onComplete){
		var _self = this;
		var handlerObj = {
				success: function(categoryTypes) {							
					if(!_self.orderPrefEntry){
						_self.getCategoryOrder(onComplete, categoryTypes);
					} else {
						categoryTypes = Utility.sortCategoryTypes(_self.orderPrefEntry, categoryTypes);
						_self.categoryTypes = dojo.isArray(categoryTypes) ? categoryTypes : [];
						_self.loaded = true;		
						onComplete(categoryTypes);					
					}
				},
				failure: function(e){
					console.derror(e);
				}
		};
		var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
		CategoryClient.getCategoryTypes(responseHandler, this.artifactType, true,false);
	},	    

	rootCategoryValuesProvider: function(item, onComplete) {	    	
		var self = this;
		this.fetchingCategories = true;			
		var handler = {
				onComplete: onComplete,
				success: function(categoriesDTO) {
					categoriesDTO = sortArrayByName(categoriesDTO);
					self.fetchingCategories = false;
					this.onComplete(categoriesDTO);
				},
				failure: function() {
					self.fetchingCategories = false;
					this.onComplete([]);
				}
		};
		var responseHandler = new ServiceResponseHandler(handler, "success", "failure");

		// always fetch the full set of categories, including the archived ones. If the user
		// switch show deleted after we've fetched w/o the archived, we can't display them. It's
		// better to fetch all one time and filter them here, rather then to refetch again
		// when the user switch show deleted.
		CategoryClient.getCategories(responseHandler, item.itemId, this.artifactType, /*fetch archived always*/true);
	},

	// provider method to return the constraint children category values given a parent category value item 
	subCategoryValuesProvider: function(childCategoryType, onComplete) {
		
		if(childCategoryType.isNew){
		//nothing to do, this is a new type
			onComplete([], false);
			return;
		}
		var ancestorValue = childCategoryType.ancestorValue;
		var protoType = this.categoryBase[childCategoryType.itemId];
		var includeAll = protoType.fullCategoryMapIncluded?false: true;
		var handler = {
				onComplete: onComplete,
				success: function(categoriesDTO) {
					categoriesDTO = categoriesDTO || {};
					categoriesDTO.allValues = sortArrayByName(categoriesDTO.allValues);
					categoriesDTO.associatedValues = sortArrayByName(categoriesDTO.associatedValues);

					this.onComplete(dojo.clone(categoriesDTO), includeAll);
				},
				failure: function() {
					this.onComplete([], includeAll);
				}
		};				
		var responseHandler = new ServiceResponseHandler(handler, "success", "failure");
		CategoryClient.getCategoryValuesExt(responseHandler, [childCategoryType.itemId], ancestorValue.itemId, includeAll);
	},
	/**
	 * Service calls End
	 **/	
});


dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryModel.CategoryModelUtils",
		null, {
	/**
	 * Utils
	 **/        
	_createCategoryTypesMap: function(){
		var len = this.categoryTypes? this.categoryTypes.length: 0;
		if(!this.categoryBase){
			this.categoryBase = {};
		}
		for(var i=0; i<len; i++){
			var categoryType = this.categoryTypes[i];
			categoryType.type = NodeTypes.CATEGORY_TYPE;
			this.categoryBase[categoryType.itemId] = dojo.clone(categoryType);    		
		}
		this._createCategoryTypesDependencies();
	},

	_createCategoryTypesDependencies: function(){
		var len = this.categoryTypes? this.categoryTypes.length: 0;    	
		for(var i=0; i<len; i++){
			var categoryType = this.categoryTypes[i];
			if(categoryType.dependsOn){
				var categoryParent = this.categoryBase[categoryType.dependsOn];
				if(!categoryParent.childTypes){
					categoryParent.childTypes = [];
				}
				categoryParent.childTypes.push(categoryType);
			}    		
		}    
	},

	_getRootTypes: function(){
		this.fullCategoryMap = {};
		var rootCategories =  [];
		var len = this.categoryTypes? this.categoryTypes.length: 0;    	
		
		for(var i=0; i<len; i++){
			var categoryType = this.categoryTypes[i];
			if(!categoryType.dependsOn && !categoryType.archived){				
				rootCategories.push(this._createNewRootElement(categoryType));
			}
		}
		return rootCategories;
	},

	_handleGetRoot: function(/*function(items)*/ onComplete){
		if (!this.root) {        	
			this._createCategoryTypesMap();
			this.root = {
					type: NodeTypes.ROOT_TYPE,
					label: "",
					children: this._getRootTypes(),
					identity: "root"
			};
		}        
		if (onComplete) {
			onComplete(this.root);
		}               
	},	
	
	/**
	 * @param item
	 * @param categoryTypesOnly
	 * WARNING: using this method will destroy children and recreate them, handle with care, this will remove all previous changes in model
	 **/
	_generateChildrenForItem: function(item, categoryTypesOnly){
		var children = [];
		if(item.type == NodeTypes.ROOT_TYPE){
			children =  this._getRootTypes();
		}else if(item.type == NodeTypes.CATEGORY_VALUE_TYPE || categoryTypesOnly){
			//values
			var childTypes = null;
			//item is ready or somehow we lost children in the way, so recreate them as a safety measure
			if(!item.isReady ||(!categoryTypesOnly && !item.children) || (categoryTypesOnly)){				
				childTypes = this._getChildTypesFromCategory(item, false, categoryTypesOnly);					
				children = childTypes || [];
			} else {	
				childTypes = !categoryTypesOnly?item.children:item.childrenHierarchyOnly;
				children  = childTypes || [];
			}						
		} else if(item.type == NodeTypes.CATEGORY_TYPE){
			//type
			if(!item.categories){
				item.categories = [];
			}
			var categories = item.categories;
			var finalCategories = categories;
			
			var typeBase = this.categoryBase[item.itemId];			
			if(!item.isReady){
				finalCategories = [];
				var len = categories?categories.length: 0;				
				for (var i=0; i<len;i++){
					var category = categories[i];					
					if(typeBase){
						var valueBase = this._searchCategoryValueFromBase(typeBase, category.itemId || category.name);
						if(!valueBase || valueBase.archived){
							continue;
						}
					}
					category.type = NodeTypes.CATEGORY_VALUE_TYPE;
					category.iconClass = CategoryValueIcon;
					if(this.isDefaultValue(category))
						category.stateIconClass = "set-default-category-icon";
					category.ancestorType = item;
					category.identity = this.identityCounter++;
					finalCategories.push(category);
				}
				item.categories = finalCategories;
			}
			children = finalCategories;
		}
		if(!this.typeHierarchyOnly){
			//force refresh every time for hierarchy only mode
			item.isReady = true;
		}
		if(!categoryTypesOnly){
			item.children = children;
		} else {
			item.childrenHierarchyOnly = children;
		}
	},
	_setMultiSelectableIconClass: function(item) {		
		var base = this.categoryBase[item.itemId || item.name];
		 base = base || item; //item may be new, for this case let's just grab the item
		
		if(base.multiSelectable || base.isMultiValued)
			item.stateIconClass = "set-multi-value-icon";
		else
			item.stateIconClass = "";
	},
	_containsItemIds: function(itemId, itemIdsArray){
		var len = itemIdsArray?itemIdsArray.length:0;
		for(var i=0;i<len;i++){
			if(itemIdsArray[i] == itemId){
				return true;
			}
		}
		return false;
	},
	_getChildTypesFromCategory: function(item, includeArchived, categoryTypesOnly){
		var finalChilds = [];
		var categoryTypeItemId = categoryTypesOnly?(item.itemId || item.name):(item.categoryType.itemId || item.categoryType.name);
		var categoryType = this.categoryBase[categoryTypeItemId];
		var childTypes =categoryType && categoryType.childTypes?categoryType.childTypes:[];
		var len = childTypes?childTypes.length:0;			
		for(var i=0; i<len; i++){					
			var childType = dojo.clone(childTypes[i]);
			if(childType.archived){
				continue;
			}
			if(!categoryTypesOnly){
				childType.ancestorValue = item;
			} else {
				childType.ancestorType = item;
			}					
			childType.iconClass = CategorySubTypeIcon + (this.typeHierarchyOnly?DisabledStyle:"");
			childType.labelClass = "category-type-label";
			this._setMultiSelectableIconClass(childType);
			childType.identity = this.identityCounter++;
			
			finalChilds.push(childType);
		}
		return finalChilds;
	},
		
	_addAllCategoryValuesToType: function(categoryType, allCategoryValues){	
		var typeBase = this.categoryBase[categoryType.itemId];		
		typeBase.fullCategoryMapIncluded = true;		
		var allValues = {};
		var len = allCategoryValues?allCategoryValues.length:0;
		for (var i=0; i< len; i++){			
			var value = allCategoryValues[i];
			value.type = NodeTypes.CATEGORY_VALUE_TYPE;
			allValues[value.itemId] = dojo.clone(value);
		}
		typeBase.allValues = allValues;
	},	
	
	_arrangeCategoryValuesWithTypes: function(categoryValues, launcherCategoryType){
		var parentType =  launcherCategoryType;
		if(!parentType.categories){
			parentType.categories = [];
		}
		
		var len = categoryValues?categoryValues.length:0;		
		for(var i=0; i<len; i++){			
			var categoryValue = categoryValues[i];			
			parentType.categories.push(categoryValue);
		}	
	},
				
	createCategoryValueDummyItem: function(valueName){
		return {
			archived: false,
			isNew: true,
			isDirty: true,
			iconClass: CategoryValueIcon,
			name: valueName,
			purged: false,
			sortable: false,
			isReady: false,
			identity: this.identityCounter++,
			type: NodeTypes.CATEGORY_VALUE_TYPE
		};		
	},
	
	createCategoryTypeDummyItem: function(typeName, parentCategoryItem){
		var item = {
			archived: false,
			isDirty: true,
			isNew: true,
			isReady: false,
			multiSelectable: false,
			iconClass: CategorySubTypeIcon,
			labelClass: "category-type-label",
			name: typeName,
			purged: false,
			required: false,			
			identity: this.identityCounter++,
			type: NodeTypes.CATEGORY_TYPE
		};
		if(parentCategoryItem && parentCategoryItem.type == NodeTypes.CATEGORY_TYPE){
			item.dependsOn = parentCategoryItem.itemId?parentCategoryItem.itemId:parentCategoryItem.name;			
		}
		return item;
	},		
	
	_getDataCategoryValue: function(item, categoryValue){
		var categoryParentTypeItemIdOrName = categoryValue.categoryType?categoryValue.categoryType.itemId: categoryValue.categoryType.name;
		if(item.itemId == categoryParentTypeItemIdOrName || item.isNew && item.name == categoryParentTypeItemIdOrName){
			var categoryValues = item.categories || [];
			var len = categoryValues.length;
			for(var i=0;i<len;i++){
				var value = categoryValues[i];
				if(value.itemId == categoryValue.itemId || categoryValue.isNew && categoryValue.name == value.name){
					return value;
				}
			} 
		} else {
			var childs = item.childTypes || [];
			var len = childs.length;			
			for(var i=0;i<len;i++){
				var child = childs[i];								
				var value = this._getDataCategoryValue(child, categoryValue);
				if(value!= null){
					return value;
				}
			}			
		} 
		return null;
	},
		
	_addCategoryTypeToParentValueType: function(parentItem, item){
		var parentType =  parentItem.ancestorType;
		return this._addCategoryTypeToParentType(parentType, item);
	},
	
	searchRepresentativeItemInHierarchy: function(itemId){
		var catType = this.categoryBase[itemId];
		if(!catType){
			return null;
		}
		var rootInGeneralMode =  this.findRootCategory(catType);
		var rootItemId = rootInGeneralMode.itemId || rootInGeneralMode.name;		
		if(rootItemId == itemId){
			return rootInGeneralMode;
		} else {
			return  this._searchRepresentativeItemFromGeneralHierarchy(catType, rootInGeneralMode);
		}
	},
	
	_addCategoryTypeToParentTypeHierarchyOnly: function(parentItem, item){
		var children = [];
		var changedItems = [];
		item.iconClass = CategorySubTypeIcon;
		if(parentItem ){			
				children = parentItem.childrenHierarchyOnly || [];			
		}
		children.push(dojo.clone(item));		
		
		item.dependsOn = parentItem.itemId || parentItem.name;				
		var rootInGeneralMode =  this.findRootCategory(item);		
		var representativeItem =  this._searchRepresentativeParentItemFromGeneralHierarchy(item, rootInGeneralMode);
		
		if(!representativeItem){			
			if(item.isRoot){
				item.isRoot = false;
				var rootItem = this.fullCategoryMap[item.itemId || item.name];
				var rootChildren = this.root.children;
				var index = dojo.indexOf(rootChildren, rootItem);
				if(index!= -1){
					rootChildren.splice(index, 1);
				}
				changedItems.push(this.root);
				this.root.childrenHierarchyOnly = this.root.children;
			}
			
			//no representative item means the category has not been opened, so we should just update the type			
			//add it to the full map			
			if(item.isNew){
				this.categoryBase[item.name] = item;
			} else {
				this.categoryBase[item.itemId] = item;
			}

			//if this is not a root, update the fullCategoryMap too, if this is a root we are done
			var parentType =  this.categoryBase[parentItem.itemId] || this.categoryBase[parentItem.name];
			if(!parentType.childTypes){
				parentType.childTypes = [];
			}
			if(parentType.isNew){
				item.dependsOnNewType = true;
			}
			var copyType = dojo.clone(item);
			copyType.ancestorType = parentItem;
			parentType.childTypes.push(item);
			parentType.hasDirtySubTypes = true;
			parentType.hasDirtyCategories = true;			
		} else {
			var changedValues = this._addCategoryTypeToParentType(representativeItem, item);
			this.changedItemsInGeneralModel = this.changedItemsInGeneralModel.concat(changedValues);
		}
		changedItems.push(parentItem);
		return changedItems;
	},
	
	_searchRepresentativeParentItemFromHierarchyOnly: function(baseType, currentLeaf){
		var children = currentLeaf.childrenHierarchyOnly || [];
		var len = children.length;
		if((currentLeaf.itemId && (currentLeaf.itemId == baseType.itemId)) || currentLeaf.name == baseType.name){
			return currentLeaf;
		} else {
			for(var i=0;i<len; i++){
				var item = this._searchRepresentativeParentItemFromHierarchyOnly(baseType, children[i]);
				if(item){
					return item;
				}
			}					
		}	
	},
	
	_searchRepresentativeParentItemFromGeneralHierarchy: function(baseType, currentLeaf){
		var children = currentLeaf.children || [];
		var len = children.length;
		if(currentLeaf.type == NodeTypes.CATEGORY_TYPE && (currentLeaf.itemId || currentLeaf.name) == baseType.dependsOn){
			return currentLeaf;			
		} else {
			for(var i=0;i<len; i++){
				var item = this._searchRepresentativeParentItemFromGeneralHierarchy(baseType, children[i]);
				if(item){
					return item;
				}
			}			
		}
		return null;
	},
	
	_addCategoryTypeToParentType: function(parentItem, item){
		var changedValues = [];	
		if(item.isNew){
			//If it is not new we assume it already has an entry in categoryBase
			this.categoryBase[item.name] = item;
		} else {
			this.categoryBase[item.itemId] = item;									
		}				
		
		if(!item.isRoot){
			var root = this.findRootCategory(item);
			this._removeSubCategoryTypeItemRecursively(item, root, changedValues);
		} else {
			var rootChildren = this.root.children || [];
			var len = rootChildren.length;
			for(var i=0;i<len;i++){
				var rootChild = rootChildren[i];
				if((!item.isNew && item.itemId == rootChild.itemId) || (item.isNew && item.name == rootChild.name)){
					rootChildren.splice(i, 1);
					break;
				}
			}
			item.isRoot = false;
			changedValues.push(this.root);
		}
		
		item.isDirty = true;
		item.dependsOn = parentItem.itemId || parentItem.name;		
		item.iconClass = CategorySubTypeIcon;
		
		//if this is not a root, update the fullCategoryMap too, if this is a root we are done
		var parentType =  this.categoryBase[parentItem.itemId] || this.categoryBase[parentItem.name];
		if(!parentType.childTypes){
			parentType.childTypes = [];
		}
		if(parentType.isNew){
			item.dependsOnNewType = true;
		}
		var copyType = dojo.clone(item);
		parentType.childTypes.push(copyType);			
		
		//need to refresh all category values
		
		var len = parentItem.children?parentItem.children.length:0;
		for(var i =0;i<len;i++){
			var child = parentItem.children[i];
			if(child.isReady){
				if(!child.children){
					child.children = [];
				}
				var type = dojo.clone(item);
				type.ancestorValue = child;
				type.identity = this.identityCounter++;
				child.children.push(type);
				changedValues.push(child);
			}						
		}
		return changedValues;
	},
	
	_addNewCategoryValueToType: function(value, type){		
		if(!type.isReady){
			this._generateChildrenForItem(type);
		}
		if(!type.categories){
			//safe guard, not sure why, but the item is ready and does not have categories array, lets create one 
			type.categories = [];
		}
		type.categories.push(value);
		
		var protoType = this.categoryBase[type.itemId] || this.categoryBase[type.name];
		
		if(value.isNew){			
			if(!protoType.allValues){
				protoType.allValues = {};
			}
			value.categoryType = {
					name:protoType.name,
					itemId: protoType.itemId
			};
			protoType.allValues[value.name]  = dojo.clone(value);
			protoType.hasDirtyCategories = true;	
		}	
		
		var valueBase = protoType.allValues[value.itemId || value.name];
		
		//Let ancestor value know that we have a subValue Added
		if(type.ancestorValue){
			var ancestorValue = type.ancestorValue;
			var ancestorTypeBase =  this.getCategoryBaseFromModelType(ancestorValue.ancestorType);
			var baseAncestorValue = this._searchCategoryValueFromBase(ancestorTypeBase, ancestorValue.itemId || ancestorValue.name);
			
			baseAncestorValue.subTypesModified = true;
			baseAncestorValue.isNewAssociated = true;
			ancestorTypeBase.hasDirtyCategories = true;			
									
			if(value.isNew){
				if(!baseAncestorValue.newValuesAdded){
					baseAncestorValue.newValuesAdded = [];
				}		
				baseAncestorValue.newValuesAdded.push(valueBase);			
			} else {
				if(!baseAncestorValue.addedValues){
					baseAncestorValue.addedValues = [];
				}		
				baseAncestorValue.addedValues.push(valueBase);
			}
		}												
		value.ancestorType = type;				
		return [type];			
	},

	_getChangedValuesRecursively: function(categoryValue, currentLeaf, changedItems, newName){
		if(!currentLeaf.isReady ){  //leaf is not ready, no need to continue  
			return;
		}		
		var baseType = this.categoryBase[categoryValue.categoryType.itemId || categoryValue.categoryType.name];
		if(currentLeaf.type == NodeTypes.CATEGORY_TYPE && currentLeaf.itemId == categoryValue.categoryType.itemId){
			var value = this._searchCategoryValueFromType(currentLeaf, categoryValue.itemId || categoryValue.name);			
			if(value) {
				if(newName){
					value.name = newName;
				}
				changedItems.push(value);				
			}
			return changedItems;
		} else if(currentLeaf && currentLeaf.dependsOn == baseType.dependsOn){
			//this is a sibling, no need to continue
			return changedItems; 
		} else {
			//this is a value, so send all types to check
			var children = currentLeaf.children; 
			var len = children?children.length:0;
			for(var i=0; i<len; i++){
				var child = children[i];
				this._getChangedValuesRecursively(categoryValue, child, changedItems, newName);
			}
		}
		return changedItems;
	},
	
	_updateValueBase: function(item){
		var typeBase =  this.categoryBase[item.categoryType.itemId || item.categoryType.name];
		var valueBase =  this._searchCategoryValueFromBase(typeBase, item.itemId || item.name);
		typeBase.hasDirtyCategories = true;
		valueBase.isDirty =  true;
		valueBase.isDefault = item.isDefault;
		if(item.removeDefault == true){
			typeBase.removeDefault = true;
			typeBase.isDirty = true;
		}
		//this needs to be done before changing name, otherwise we won't have any means for identify the categoryValue for new created items
		var root = this.findRootCategory(item);
		var changedItems = [];
		changedItems = this._getChangedValuesRecursively(item, root, changedItems, item.newName);		
		
		if(item.newName){
			if(valueBase.isNew){
				delete typeBase.allValues[valueBase.name];
				typeBase.allValues[item.newName] =  valueBase;
			}
			valueBase.name = item.newName;
			item.newName = false;			
		}
		return changedItems;
	},
	
	_updateCategoryTypeBase: function(item){
		var typeBase = this.categoryBase[item.itemId || item.name];
		typeBase.isDirty = true;
		typeBase.isRequired = typeBase.required = item.isRequired;
		typeBase.isMultiValued = typeBase.multiSelectable = (item.isMultiValued || item.multiSelectable);
		this._setMultiSelectableIconClass(item);
		var oldName =  item.name;
		var newName =  item.newName; //if any, can be null		
		if(newName){
			typeBase.newName = newName;
		}
		//this needs to be done before changing name, otherwise we won't have any means for identify the categoryValue for new created items
		var root = this.findRootCategory(item);
		var changedItems = [];
		
		if(item.isRoot){			
			changedItems.push(item);
		} else {
			changedItems = this._getChangedSubTypesRecursively(typeBase, root, changedItems);
		}
		
		if(newName){
			item.name = newName;
			if(typeBase.isNew){
				delete this.categoryBase[oldName];
				this.categoryBase[newName] = typeBase;
				if(typeBase.isRoot){
					if(typeBase.newName){
						this._updateCategoryTypeNameForValues(typeBase, item);
					}	
					var rootItem = this.fullCategoryMap[oldName]; 
					delete this.fullCategoryMap[oldName];
					this.fullCategoryMap[newName] = rootItem;
				}
			}			
			typeBase.name = newName;
			delete typeBase.newName;
			item.newName = false;

			//update childTypes
			if(typeBase.dependsOn){
				var parentType = this.categoryBase[typeBase.dependsOn];
				var childTypes = parentType.childTypes;
				var len = childTypes?childTypes.length:0;
				for(var i=0;i<len;i++){
					var childType = childTypes[i];
					var itemIdOrName = childType.itemId || childType.name;
					var itemIdOrNameChild =  typeBase.itemId  || oldName;
					if(itemIdOrName == itemIdOrNameChild){
						childType.name = newName;
						break;
					}
				}
			}
		}				
		return changedItems;
	},
	
	_getChangedSubTypesRecursively: function(baseType, currentLeaf, changedItems){
		var currentLeafItemIdOrName = currentLeaf.itemId || currentLeaf.name;
		var baseTypeItemIdOrName = baseType.itemId || baseType.name;
		if (currentLeaf.type ==  NodeTypes.CATEGORY_TYPE && currentLeafItemIdOrName == baseTypeItemIdOrName){
			
			currentLeaf.multiSelectable = baseType.multiSelectable;
			this._setMultiSelectableIconClass(currentLeaf);
			currentLeaf.required = baseType.required;
			if(baseType.multiSelectable){
				this._setMultiSelectableIconClass(currentLeaf);
			}
			if(baseType.newName){
				currentLeaf.name = baseType.newName;
				this._updateCategoryTypeNameForValues(baseType, currentLeaf);
			}							
			changedItems.push(currentLeaf);		
			return;
		} else if(currentLeaf.type == NodeTypes.CATEGORY_TYPE && currentLeaf.dependsOn == baseType.dependsOn){
			//sibling type, return;
			return;
		}
		
		var children = currentLeaf.children; 
		var len = children?children.length:0;
		for(var i=0; i<len; i++){
			var child = children[i];
			this._getChangedSubTypesRecursively(baseType, child, changedItems);

		}
		
		return changedItems;
	},
	
	_removeValueFromValueBaseReferences: function(valueBase, valueToRemove){
		var valuesAdded = valueBase.addedValues || [];
		var newValuesAdded = valueBase.newValuesAdded || [];
		var len  = valuesAdded.length;
		var itemIdOrName = valueToRemove.itemId || valueToRemove.name;
		
		var changed = false;
		
		for(var i=0;i<len; i++){
			var valueAdded = valuesAdded[i];
			var itemIdOrNameValueAdded = valueAdded.itemId ||  valueAdded.name;			
			if(itemIdOrNameValueAdded == itemIdOrName){
				valuesAdded.splice(i, 1);
				changed = true;
				break;
			}
		}
		
		len  = newValuesAdded.length;
		for(var i=0;i<len; i++){
			var newValueAdded = newValuesAdded[i];
			var itemIdOrNameValueAdded = newValueAdded.itemId ||  newValueAdded.name;			
			if(itemIdOrNameValueAdded == itemIdOrName){
				newValuesAdded.splice(i, 1);
				changed = true;
				break;
			}
		}
		return changed;
	},
	
	_removeCategoryValueFromType: function(item, updatingReferences){		
		var type = item.ancestorType;
		var children = type.children;
		
		//first lets remove the category from view
		var index = dojo.indexOf(children, item);
		if(index!= -1){
			//remove from category data
			type.categories.splice(index, 1);
		}
		
		var categoryTypeBase = this.categoryBase[type.itemId || type.name];
		var categoryValueBase =  this._searchCategoryValueFromBase(categoryTypeBase, item.itemId ||  item.name);
		var toBeDeleted = item.toBeDeleted;				
		
		if(type.ancestorValue){
			//this is a subType, and we should only remove the reference from  the value			
			var parentTypeBase = this.categoryBase[type.ancestorValue.categoryType.itemId || type.ancestorValue.categoryType.name];
			var ancestorValueBase = this._searchCategoryValueFromBase(parentTypeBase, type.ancestorValue.itemId || type.ancestorValue.name);
			
			var changed = this._removeValueFromValueBaseReferences(ancestorValueBase, categoryValueBase);											
			if(!changed){				
				if(!ancestorValueBase.removedChildren){
					ancestorValueBase.removedChildren = [];				
				}		
				ancestorValueBase.removedChildren.push(categoryValueBase);
				ancestorValueBase.subTypesModified = true;
			}
			parentTypeBase.hasDirtyCategories = true;
		}		
		if(toBeDeleted || (!type.ancestorValue && !updatingReferences)){			
			var changedItems = [];	
			changedItems.push(type);
			//remove it from view
			var root = this.findRootCategory(item); 
			this._removeCategoryValueFromSubTypeRecursively(categoryValueBase, categoryTypeBase, root, changedItems);
			//we either come from root or this is a subType but the user selected to remove completely
			if(categoryValueBase.isNew){
				delete categoryTypeBase.allValues[categoryValueBase.name];
			} else {
				categoryTypeBase.hasDirtyCategories = true;
				categoryValueBase.archived = true;
				categoryValueBase.isDirty = true;
			}
			return changedItems;
		}else {
			//we just removed the value association, so only update the type and we are done
			return [type];
		}
	},
	
	_removeCategoryValueFromSubTypeRecursively: function(valueToRemove, baseType, currentLeaf, changedItems){
		if(!currentLeaf.isReady ){  //leaf is not ready, no need to continue  
			return;
		}		
				
		if(currentLeaf.type == NodeTypes.CATEGORY_TYPE && (currentLeaf.itemId || currentLeaf.name) == (baseType.itemId || baseType.name)){
			var value = this._searchCategoryValueFromType(currentLeaf, (valueToRemove.itemId || valueToRemove.name));			
			if(!value){
				//break cases, value was not found, so probably is alreasdy removed or not contained at all 
				return;
			}
			var changed = this._removeCategoryValueFromType(value, true);
			var len = changed?changed.length:0;
			for(var i=0; i<len; i++){
				changedItems.push(changed[i]);
			}
			return;			
		}  else  if(currentLeaf && currentLeaf.dependsOn == baseType.dependsOn){
			//this is a sibling, no need to continue
			return; 
		} else {
			//this is a value, so send all types to check
			var children = currentLeaf.children; 
			var len = children?children.length:0;
			for(var i=0; i<len; i++){
				var child = children[i];
				this._removeCategoryValueFromSubTypeRecursively(valueToRemove,baseType,  child, changedItems);
			}
		}
		
	},
	
	_searchCategoryValueFromBase: function(baseType, categoryValueItemIdOrName){
		var categories = baseType.allValues || {};	
		return categories[categoryValueItemIdOrName];		
	},		
	_searchCategoryValueFromType: function(type, categoryValueItemIdOrName){
		var categories =type.categories;
		var len = categories.length;
		for(var i=0; i<len; i++){
			var value = categories[i];
			if(value.itemId == categoryValueItemIdOrName || value.name == categoryValueItemIdOrName){
				return value;
			}
		}
	},	
	
	_updateCategoryTypeNameForValues: function(baseType, item){				
		var values = item.children;
		var len = values?values.length:0;
		for(var i=0;i<len;i++){
			var value = values[i];
			value.categoryType = {
					itemId: baseType.itemId,
					name: baseType.newName
			};
			value.ancestorType = item;
		}		
	},
	
	_removeSubCategoryTypeItemRecursively: function(typeToRemove, currentLeaf, changedItems){		
		if(!currentLeaf.isReady){
			return;
		}
		if(currentLeaf.type == NodeTypes.CATEGORY_TYPE && (currentLeaf.itemId || currentLeaf.name) == typeToRemove.dependsOn ||
			currentLeaf.type == NodeTypes.CATEGORY_VALUE_TYPE && (currentLeaf.categoryType.itemId || currentLeaf.categoryType.name) == typeToRemove.dependsOn){
			var children = currentLeaf.children; 
			var len = children?children.length:0;
			var changed = false;
			for(var i=0; i<len; i++){
				var child = children[i];
				if((child.itemId || child.name) == (typeToRemove.itemId || typeToRemove.name)){
					changed = true;					
					children.splice(i, 1);
					break;
				} else {
					this._removeSubCategoryTypeItemRecursively(typeToRemove, child, changedItems);
				}
			}		
			if(changed){
				changedItems.push(currentLeaf);
			}
		} else {
			var children = currentLeaf.children; 
			var len = children?children.length:0;
			for(var i=0; i<len; i++){
				var child = children[i];
				this._removeSubCategoryTypeItemRecursively(typeToRemove, child, changedItems);
			}
		}
		return changedItems;
	},	
	
	_removeSubCategoryType: function(item){
		//remove it from the categoryMap data structure
		var childType = this.categoryBase[item.itemId || item.name];
		var protoType = this.categoryBase[childType.dependsOn];		
		var changedItems =  [];
		var root = this.findRootCategory(item);
		this._removeSubCategoryTypeItemRecursively(item, root, changedItems);
		childType.dependsOn = null;
		childType.isDirty = true;
		childType.archived = item.toBeDeleted;		
		
		var len = protoType.childTypes? protoType.childTypes.length:0;
		for(var i=0; i<len; i++){
			var protoChildType = protoType.childTypes[i];
			if(protoChildType.itemId == childType.itemId || protoChildType.name == childType.name){				
				protoType.childTypes.splice(i, 1);
				break;
			}
		}						
						
		//item should return to root, so we must clear up all flags and force to recreate children
		if(childType && !item.toBeDeleted){
			item.ancestorValue = null;
			item.isReady = false;
			item.categories = null;
			this._createNewRootElement(item);		
			childType.isRoot = true;
			changedItems.push(this.root);			
		}		
				
		return changedItems;						
	},
	
	_searchRepresentativeItemFromGeneralHierarchy: function(baseType, currentLeaf){
		var currentLeafBase = this.categoryBase[currentLeaf.itemId || currentLeaf.name];
		var children = currentLeaf.children || [];
		var len = children.length;		
		if(currentLeaf.type == NodeTypes.CATEGORY_TYPE && ((currentLeafBase.itemId || currentLeafBase.name) == (baseType.itemId || baseType.name))){
			return currentLeaf;			
		} else {
			for(var i=0;i<len; i++){
				var item = this._searchRepresentativeItemFromGeneralHierarchy(baseType, children[i]);
				if(item){
					return item;
				}
			}
			
		}
		return null;
	},
	
	_removeSubCategoryTypeHierarchyOnly: function (item, parentItem){						
		var parent =  item.ancestorType || parentItem;		
		var childCatType = this.categoryBase[item.itemId || item.name];		
		var rootInGeneralMode =  this.findRootCategory(childCatType);
		var changedItems = [];
		if (!parent) {
			var itemBase = this.categoryBase[item.itemId || item.name];
			var parentCatProtoType = this.categoryBase[itemBase.dependsOn];
			//as a last resort we should try to find the parent manually, otherwise we will just ignore the parent update
			parent = this._searchRepresentativeParentItemFromHierarchyOnly(parentCatProtoType, rootInGeneralMode); 			
		}		
		if(parent){
			var children = parent.childrenHierarchyOnly || [];						
			if(parent && parent.childrenHierarchyOnly){
				//assure item is ready
				changedItems.push(parent);
			}
			var len = children.length;
			for(var i=0;i<len;i++){
				var child = children[i];
				if((child.itemId || child.name) == (item.itemId || item.name)){
					children.splice(i,1);
					break;
				}
			}						
		}	
		var representativeItem =  this._searchRepresentativeItemFromGeneralHierarchy(item, rootInGeneralMode);		
		if(!representativeItem){
			var catType = this.categoryBase[childCatType.dependsOn];			
			childCatType.dependsOn = null;
			childCatType.isDirty = true;
			childCatType.archived = item.toBeDeleted;			
			
			var len = catType.childTypes? catType.childTypes.length:0;
			for(var i=0; i<len; i++){
				var protoChildType = catType.childTypes[i];
				if((protoChildType.itemId ||  protoChildType.name) == (childCatType.itemId || childCatType.name)){				
					catType.childTypes.splice(i, 1);
					break;
				}
			}				
			//item should return to root, so we must clear up all flags and force to recreate children
			if(childCatType && !item.toBeDeleted){
				item.isReady = false;
				item.categories = null;
				this._createNewRootElement(item);		
				childCatType.isRoot = true;						
			}	
		}else {
			var changedItemsGeneral = this._removeSubCategoryType(representativeItem);
			this.changedItemsInGeneralModel = this.changedItemsInGeneralModel.concat(changedItemsGeneral);
		}			
		if(!item.toBeDeleted){
			changedItems.push(this.root);
		}
		return changedItems;
	},
	
	_categoryTypeBaseMixIntoItem: function(categoryBase, item){
		item.archived = categoryBase.archived;
		item.itemId = categoryBase.itemId;
		item.name = categoryBase.name;
		item.purged = categoryBase.purged;
		item.required = categoryBase.required;
		item.type = categoryBase.type;
		item.multiSelectable = item.isMultiValued = categoryBase.multiSelectable;
		item.hasDirtyCategories = categoryBase.hasDirtyCategories;
		return item;
	},
	
	_createNewRootElement: function(item){
		var newItem = {};
		if(!item.isNew){
			var categoryType = this.categoryBase[item.itemId || item.name];
			categoryType.isRoot =  true;
			this._categoryTypeBaseMixIntoItem(categoryType, newItem);
			item = newItem;
		} else {
			newItem = item;
		}							
		newItem.iconClass = CategoryRootTypeIcon;
		newItem.labelClass = "category-type-root-label";						
		this._setMultiSelectableIconClass(newItem);		
		if(item.identity && item.identity > 0){
			newItem.identity = item.identity;
		} else {
			newItem.identity = this.identityCounter++;
		}
		
		if(this.root && this.root.children){
			this.root.children.push(newItem);
		}
		this.fullCategoryMap[newItem.itemId || newItem.name] = newItem;	
		return newItem;
	},
	
	_newRootItem: function(newItem){
		var type  = this._createNewRootElement(newItem);
		type.isRoot = true;
		this.categoryBase[type.name] = dojo.clone(type);
		return [this.root];
	},
	
	_removeRootCategoryType: function(item){
		
		var children = this.root.children;
		var index =  dojo.indexOf(children, item);
		if(index!=-1){			
			this.root.children.splice(index, 1);
			var protoType =  this.categoryBase[item.itemId || item.name];
			if(protoType.isNew){
				delete this.categoryBase[item.name];
			} else {
				protoType.isDirty = true;
				protoType.archived = item.toBeDeleted;
				protoType.isRoot = false;
			}
		}										
		return [this.root];
	},
	
	findSubCategoryInGeneralModel: function(item){
		return this.categoryBase[item.itemId]  || this.categoryBase[item.name];	
	},
	
	findRootCategory: function(item){
		if(item.type!= NodeTypes.CATEGORY_VALUE_TYPE && !item.dependsOn){			
			return this.fullCategoryMap[item.itemId || item.name];
		} else if (item.type == NodeTypes.CATEGORY_TYPE){
			var type =  this.categoryBase[item.dependsOn];
			return (this.findRootCategory(type));
		}else if (item.type == NodeTypes.CATEGORY_VALUE_TYPE){
			var type = this.categoryBase[item.categoryType.itemId || item.categoryType.name];
			return (this.findRootCategory(type));
		}
		// should not happen 
		return {};		
	},
	
	getDirectChangedSubValuesFromValue: function(item){
		var root = this.findRootCategory(item);
		var valueData =  this._getDataCategoryValue(root, item);
		var childTypes = valueData.children || [];		
		var len =  childTypes.length;
		var changed = [];
		for(var i=0;i<len; i++){
			var catType = childTypes[i];
			var categories = catType.children || [];
			var len = categories.length;
			for(var j=0;j<len; j++){
				var category = categories[j];
				if(category.isNew || category.isNewAssociated){
					changed.push(category);
				}
			}
		}
		return changed;
	},
	
	getAllCategoryValuesForType: function(categoryTypeItemId){
		var type = this.categoryBase[categoryTypeItemId];
		var allCategoriesMap = type.allValues;
		var allCategoriesList = [];
		for(var key in allCategoriesMap){
			if(allCategoriesMap.hasOwnProperty(key)){
				allCategoriesList.push(allCategoriesMap[key]);
			}
		}				
		return allCategoriesList;						
	},
	
	getAllCategoryTypes: function(){
		var categoryTypes = [];		
		for (var key in this.categoryBase) {
		    if (this.categoryBase.hasOwnProperty(key)) {
		    	var categoryType = this.categoryBase[key];
		    	categoryTypes.push(categoryType);
		    }
		}
		return categoryTypes;
	},
	
	getAllCategoryTypesUnchanged: function(){
		var categoryTypesUnchanged = [];		
		for (var key in this.categoryTypes) {
		    if (this.categoryTypes.hasOwnProperty(key)) {
		    	var categoryType = this.categoryTypes[key];
		    	categoryTypesUnchanged.push(categoryType);
		    }
		}
		return categoryTypesUnchanged;
	},
	
	getCategoryTypeFromBase:function(itemId){
		return this.categoryBase[itemId];
	},
	
	getCategoryBaseFromModelType: function(type){		
		return this.categoryBase[type.itemId || type.name];
	},
		
	
	isCategoryType: function(item) {
		return item.type == NodeTypes.CATEGORY_TYPE;
	},

	isCategoryValue: function(item) {
		return item.type == NodeTypes.CATEGORY_VALUE_TYPE;
	},		
	
	getAllItemsChangedInGeneralModel: function(){
		return this.changedItemsInGeneralModel;
	},
	
	setHierarchyOnlyMode: function(typeHierarchyOnly){		
		this.typeHierarchyOnly = typeHierarchyOnly;
		if(this.typeHierarchyOnly){
			//clean up  			
			this.changedItemsInGeneralModel =  [];			
		}
		this._addRemoveDisabledStyleFromRootElements();
	},
	
	getDirtyTypes: function(){
		var dirtyCategoryTypes = [];		
		for (var key in this.categoryBase) {
		    if (this.categoryBase.hasOwnProperty(key)) {
		    	var categoryType = this.categoryBase[key];
		    	if(categoryType.isDirty || categoryType.hasDirtyCategories){
		    		dirtyCategoryTypes.push(categoryType);
		    	}
		    }
		}
		return dirtyCategoryTypes;
	},
	
	getDataModel: function(){
		return this.fullCategoryMap;
	},
	
	organizeSubCategoriesByType: function(category){
		var map = {				
		};
		var addedValues = category.addedValues || [];
		var newValuesAdded = category.newValuesAdded || [];
		var removedValues = category.removedChildren || [];
		
		var len = addedValues.length;
		for(var i=0;i<len;i++){
			var addedValue = addedValues[i];
			var itemType = addedValue.categoryType || addedValue.ancestorType;
			var typeItemId = itemType.itemId || itemType.name;
			if(!map[typeItemId]){
				map[typeItemId] = {};
			}
			if(!map[typeItemId].addedValues){
				map[typeItemId].addedValues = [];				
			}
			map[typeItemId].addedValues.push(addedValue);
		}
		
		len = newValuesAdded.length;
		for(var i=0;i<len;i++){
			var newValue = newValuesAdded[i];
			var itemType = newValue.categoryType || newValue.ancestorType;
			var typeItemId = itemType.itemId || itemType.name;
			if(!map[typeItemId]){
				map[typeItemId] = {};
			}
			if(!map[typeItemId].newValues){
				map[typeItemId].newValues = [];
			}
			map[typeItemId].newValues.push(newValue);
		}
		
		len = removedValues.length;
		for(var i=0;i<len;i++){
			var removedValue = removedValues[i];
			var itemType = removedValue.categoryType || removedValue.ancestorType;
			var typeItemId = itemType.itemId || itemType.name;
			if(!map[typeItemId]){
				map[typeItemId] = {};
			}
			if(!map[typeItemId].removedValues){
				map[typeItemId].removedValues = [];
			}
			map[typeItemId].removedValues.push(removedValue);
		}				
		
		var finalCategoryTypeMap = [];		
		for(var key in map){
			if(map.hasOwnProperty(key)){
				var categoryType = map[key];
				categoryType.itemId  =  key;
				finalCategoryTypeMap.push(categoryType);
			}
		}
		
		
		return finalCategoryTypeMap;
	},
	
	_newItemHierarchyOnly:function(/*String*/ itemName , parentItem, isSubCategory){
		if(parentItem && isSubCategory){
			var NewItem = this.createCategoryTypeDummyItem(itemName, parentItem);			
			var changedItems = this._addCategoryTypeToParentTypeHierarchyOnly(parentItem, NewItem);
			return changedItems?changedItems:parentItem;
		} else if(!parentItem){
			//new root
			var newItem = this.createCategoryTypeDummyItem(itemName, null);
			return this._newRootItem(newItem);
		}
	},
	
	_newItem: function(/*String*/ itemName , parentItem, isSubCategory){ // Creates a new item
		if(parentItem && isSubCategory){
			//we are adding a category type			
			var NewItem = this.createCategoryTypeDummyItem(itemName, parentItem);
			if(parentItem.type == NodeTypes.CATEGORY_TYPE ){
				return this._addCategoryTypeToParentType(parentItem, NewItem);
			} else if(parentItem.type == NodeTypes.CATEGORY_VALUE_TYPE){
				return this._addCategoryTypeToParentValueType(parentItem, NewItem);
			}
		} else if(parentItem && parentItem.type == NodeTypes.CATEGORY_TYPE ){
			//we are adding a category Value
			var category = this.createCategoryValueDummyItem(itemName);
			return this._addNewCategoryValueToType(category, parentItem);
		} else if(!parentItem && !isSubCategory){
			//we want to create a new root
			var newItem = this.createCategoryTypeDummyItem(itemName, null);
			return this._newRootItem(newItem);
		}
	},
	
	_removeItem: function(item, parentItem/*ignored in general view*/){
		if(item.type == NodeTypes.CATEGORY_VALUE_TYPE){			
			return this._removeCategoryValueFromType(item);
		} else if(!this.categoryBase[item.itemId || item.name].isRoot){ 			
			return this._removeSubCategoryType(item);
		} else {
			return this._removeRootCategoryType(item);
		}
	}, 
	
	_removeItemHierarchyOnly: function(item, parentItem){
		var baseType = this.categoryBase[item.itemId || item.name];
		if(!baseType.isRoot){ 			
			return this._removeSubCategoryTypeHierarchyOnly(item, parentItem);
		} else {
			return this._removeRootCategoryType(item);
		}
	},
	
	_updateItemHierarchyOnly: function(item){
		var rootInGeneralMode =  this.findRootCategory(item);				
		var representativeItem =  this._searchRepresentativeItemFromGeneralHierarchy(item, rootInGeneralMode);
		if(!representativeItem){
			var typeBase = this.categoryBase[item.itemId || item.name];
			typeBase.isDirty = true;
			typeBase.isRequired = typeBase.required = item.isRequired;
			typeBase.isMultiValued = typeBase.multiSelectable = item.isMultiValued;

			var oldName =  item.name;
			var newName =  item.newName; //if any, can be null		
			if(newName){
				typeBase.newName = newName;
			}
			if(newName){
				item.name = newName;
				if(typeBase.isNew){
					delete this.categoryBase[oldName];
					this.categoryBase[newName] = typeBase;
					if(typeBase.isRoot){
						if(typeBase.newName){
							this._updateCategoryTypeNameForValues(typeBase, item);
						}	
						var rootItem = this.fullCategoryMap[oldName]; 
						delete this.fullCategoryMap[oldName];
						this.fullCategoryMap[newName] = rootItem;
					}
				}			
				typeBase.name = newName;
				delete typeBase.newName;
				item.newName = false;

				//update childTypes
				if(typeBase.dependsOn){
					var parentType = this.categoryBase[typeBase.dependsOn];
					var childTypes = parentType.childTypes;
					var len = childTypes?childTypes.length:0;
					for(var i=0;i<len;i++){
						var childType = childTypes[i];
						var itemIdOrName = childType.itemId || childType.name;
						var itemIdOrNameChild =  typeBase.itemId  || oldName;
						if(itemIdOrName == itemIdOrNameChild){
							childType.name = newName;
							break;
						}
					}
				}
			}				
		} else {			
			representativeItem.isRequired = item.isRequired;
			representativeItem.isMultiValued = item.isMultiValued;
			representativeItem.newName = item.newName;
			representativeItem.name = item.name;
			if(item.newName){
				item.name = item.newName;
			}
			var changedItems = this._updateCategoryTypeBase(representativeItem);				
			this.changedItemsInGeneralModel = this.changedItemsInGeneralModel.concat(changedItems);
		}
		return [item];		
	},
	_updateItem: function(item){
		if(item.type == NodeTypes.CATEGORY_VALUE_TYPE){
			return this._updateValueBase(item);
		} else if (item.type == NodeTypes.CATEGORY_TYPE){
			return this._updateCategoryTypeBase(item);
		} else {
			//cry in a corner
		}
	},
	
	_mixInCategoryValue: function(value, item){
		item.archived = value.archived;
		item.isDirty = value.isDirty;
		item.isNew = value.isNew; 
		item.isReady = value.isReady;
		item.itemId = value.itemId;
		item.name = value.name;
		item.type = value.type;
		item.categoryType = value.categoryType;
		return item;
	},
	
	_addRemoveDisabledStyleFromRootElements: function(){
		var children = this.root?this.root.children:[];
		var iconStyle = (this.typeHierarchyOnly?DisabledStyle:"");  
		var len = children? children.length: 0;
		for(var i=0;i<len;i++){
			var child = children[i];
			child.iconClass = CategoryRootTypeIcon + iconStyle;
		}
	},
	
	_associateExistingValueInParentSubType: function(parentItem, existingItem){
		var typeBase = this.categoryBase[parentItem.itemId || parentItem.name];
		var valueBase = typeBase.allValues[existingItem.itemId || existingItem.name];
		var clone = dojo.clone(valueBase);
		var dummy = this.createCategoryValueDummyItem(valueBase.name);
		var newValue = this._mixInCategoryValue(clone, dummy);
		return this._addNewCategoryValueToType(newValue, parentItem);
	},
	/*
	 * parentItem: destination item, where the existing item will be moved to (must be an actual item from the general view(not just a base))
	 * existingItem: can be a base, the element that will be moved
	 * fromParent: parent Item, which category type will be removed from(optional) 
	 * */
	_associateExistingTypeInParentType: function(parentItem, existingItem, fromParent){
		if(fromParent){
			var fromItemIdOrName = fromParent.itemId || fromParent.name;
			var parentItemIdOrName = parentItem.itemId || parentItem.name;
			if (fromItemIdOrName === parentItemIdOrName) {
				//nothing to do, no readdings to the same parent please
				return [];
			}		
		}
		var changedItems = [];					
		var typeBase = this.categoryBase[existingItem.itemId || existingItem.name];		
		changedItems = this.removeItem(existingItem, fromParent);
		var clone = dojo.clone(typeBase);
		var dummy = this.createCategoryTypeDummyItem(typeBase.name);
		var newValue = this._categoryTypeBaseMixIntoItem(clone, dummy);		
		newValue.isRoot = (!typeBase.dependsOn);
		newValue.isNew = typeBase.isNew;
		newValue.required = typeBase.required;
		newValue.allValues = typeBase.allValues;
		newValue.childTypes = typeBase.childTypes;		
		if(existingItem.identity >0){
			newValue.identity = existingItem.identity;
		}
		if(!this.typeHierarchyOnly){
			changedItems = changedItems.concat(this._addCategoryTypeToParentType(parentItem, newValue));
		} else {
			changedItems = changedItems.concat(this._addCategoryTypeToParentTypeHierarchyOnly(parentItem, newValue));
		}
		return changedItems;
	},
	
	_indexOfItemInRoot: function(item){
		var len = (this.root && this.root.children)?this.root.children.length:0;
		for(var i=0;i<len;i++){
			var rootItem = this.root.children[i];
			if((rootItem.name || rootItem.itemId ) ==  (item.name || item.itemId)){
				return i;
			}
		}
		return -1;
	},
	
	/*
	 * Moves an item pointed in itemToMoveIndex to a new position above destinationIndex in root
	 */
	_moveItemInRootByPosition: function(itemToMoveIndex, destinationIndex){
		if(itemToMoveIndex!=-1 && destinationIndex!=-1){
			var childrenInRoot = this.root.children;
			var moveItem = childrenInRoot[itemToMoveIndex];
			childrenInRoot.splice(itemToMoveIndex, 1);
			childrenInRoot.splice(destinationIndex, 0, moveItem);
		}
	},
	
	isCategoryTypeRoot: function(item){
		var type = this.categoryBase[item.itemId || item.name];
		if(type){
			return type.isRoot;
		}
		return false;
	},		
			
	/*
	 * Moves an item (itemToMove) to a new position above destinationItem in root
	 */
	moveItemInRootbyItem: function(itemToMove, destinationItem){
		var destinationIndex = this._indexOfItemInRoot(destinationItem);
		var itemToMoveIndex = this._indexOfItemInRoot(itemToMove);
		this._moveItemInRootByPosition(itemToMoveIndex, destinationIndex);		
	},
	
	moveUpItemInRoot: function(itemToMove){
		var itemToMoveIndex = this._indexOfItemInRoot(itemToMove);
		this._moveItemInRootByPosition(itemToMoveIndex, itemToMoveIndex - 1);		
	},
	
	moveDownItemInRoot: function(itemToMove){
		var itemToMoveIndex = this._indexOfItemInRoot(itemToMove);
		this._moveItemInRootByPosition(itemToMoveIndex, itemToMoveIndex + 1);		
	},
		
	//returns true if item is the first displayed in root 
	itemIsFirst: function(item){
		var index = this._indexOfItemInRoot(item);
		var len = (this.root && this.root.children)?this.root.children.length:-2;
		if(len > 1 && index == 0){ 
			return true;
		}
		return false;
	},
	
	//returns true if item is the last displayed in root 
	itemIsLast: function(item){
		var index = this._indexOfItemInRoot(item);
		var len = (this.root && this.root.children)?this.root.children.length:-2;
		if(len > 1 && index == len -1){ 
			return true;
		}
		return false;
	},
	
	getRootCategoryTypesOrder: function(){
		var root = this.root;
		var types = [];
		var len = root && root.children?root.children.length:0;
		for(var i=0;i<len;i++){
			types.push(root.children[i].name);
		}				    	
    	return {order:types};		
	},
	
	valueExistsInParentType: function(categoryType, valueName){
		if(!categoryType || !valueName){
			return;
		}
		var values = categoryType.categories;
		var len = values?values.length:0;
		for(var i=0;i<len;i++){
			if(values[i].name == valueName)
				return true;
		}
		return false;
	}
		
	/**
	 * --- Helpers end --- 
	 */  
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryTreeModel",
		[com.ibm.asq.common.web.ui.internal.widgets.navigator.AbstractModel,
		 com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryModelDataProvider, 
		 com.ibm.rqm.planning.web.ui.internal.view.administration.categories.CategoryModel.CategoryModelUtils],{       

	labelEmptyContent: MessageUtil.getMessage(MessageBundle.COMMON, "labelNoCategories"),        

	constructor: function(args){
		dojo.mixin(this, args);		
		this.identityCounter = 1; //just to assure we have a different leaf identity each time
	},	

	isEmpty: function() {
		var root = this.root;
		if(root && root.children) {
			return root.children.length == 0;
		}
		return true;    
	},    
	
	isDefaultValue: function(item) {
		if (item.type == NodeTypes.CATEGORY_VALUE_TYPE) {
			if (item.categoryType && item.categoryType.itemId) {
				var catType = this.categoryBase[item.categoryType.itemId || item.categoryType.name];
				var allowDefault = catType && (catType.dependsOn == undefined || catType.dependsOn == null);
				// suppress showing the default value if the type is a child
				// type. We don't allow setting default values in child types. But
				// the hierarchy can be build on the fly. If a default value was 
				// set, we don't want to show it.
				if (allowDefault) {
					return catType.defaultCategoryId == item.itemId;
				}
			}
		}
		return false;
	},

	/*
	 * TreeModel
	 */                       
	getRoot: function(/*function(items)*/ onComplete){
		// The model doesn't have the category types yet, we need 
		// to fetch them.
		if(this.loaded) {
			onComplete(this.root);
			return;
		}
		if(this.categoryTypesProvider && !this.categoryTypes) {
			this.categoryTypesProvider(dojo.hitch(this, function(categoryTypes) {
				this.categoryTypes = categoryTypes;
				this._handleGetRoot(onComplete);
			}));
			return;
		}
		this._handleGetRoot(onComplete);
	},
	// dijit.tree.Model  API
	getChildren: function(/*dojo.data.Item*/parentItem, /*function(items)*/ onComplete){    	
		// handle category type & category value type
		var children = [];
		
		if (parentItem.type == NodeTypes.CATEGORY_VALUE_TYPE || (this.typeHierarchyOnly && parentItem.type != NodeTypes.ROOT_TYPE)){			
			this._generateChildrenForItem(parentItem, this.typeHierarchyOnly);
			children = !this.typeHierarchyOnly?parentItem.children:parentItem.childrenHierarchyOnly;			
		} else if (parentItem.type == NodeTypes.CATEGORY_TYPE) {
			var categoryType = parentItem;		

			if(!categoryType.categories){
				if(!categoryType.ancestorValue){  //root category
					if(!categoryType.isNew){
						this.rootCategoryValuesProvider(parentItem, dojo.hitch(this, function(categoriesDTO){
							var baseType = this.categoryBase[parentItem.itemId] || this.categoryBase[parentItem.name];
							this._addAllCategoryValuesToType(baseType, dojo.clone(categoriesDTO));
							//need to generate children
							categoryType.isReady = false;
							categoryType.categories = categoriesDTO;
							this._generateChildrenForItem(categoryType);
							onComplete(categoryType.children);						
						}));
						return;
					} else{
						onComplete(categoryType.children || []);
					}			
				} else { //subcategory					
					var callback = dojo.hitch(this, function(categoriesDTO, includeAll){
						var categories =  categoriesDTO.associatedValues;						
						if(includeAll){
							var allCategories  = categoriesDTO.allValues;
							this._addAllCategoryValuesToType(parentItem, allCategories);
						}
						this._arrangeCategoryValuesWithTypes(categories, parentItem, includeAll);												
						this._generateChildrenForItem(parentItem);						
						onComplete(parentItem.children);
					});
					if(!parentItem.ancestorValue.isNew){																
						this.subCategoryValuesProvider(parentItem, callback);
						return;
					} else {
						if(!parentItem.isReady &&  ((!this.typeHierarchyOnly && !parentItem.children) || (this.typeHierarchyOnly && !this.childrenHierarchyOnly))){
							this._generateChildrenForItem(parentItem);	
						}							
						onComplete(parentItem.children || []);
						return;
					}					
				}
			} else {				
				this._generateChildrenForItem(categoryType);
				children = parentItem.children;
			} 			
		} else if (parentItem.type ==  NodeTypes.ROOT_TYPE){

			if(this.root){
				children = this.root.children;
			} else {				
				this.getRoot(dojo.hitch(this, function(){					
					onComplete(this.root.children);	
					return;			
				}));
			}			

		}
		onComplete(children);		
	},
	
	// dijit.tree.Model  API
	getIdentity: function(/* item */ item){		
		return item.identity;
	},	

	// Summary:
	//		Move or copy an item from one parent item to another.
	//		Used in drag & drop
	//
	pasteItem: function(item, from, to) { // Other args obscured to simplify...
		var isItemCategoryValue = this.isCategoryValue(item);
		if (isItemCategoryValue) {
			if (from.isRoot && to.isRoot) {
				// Moving values between roots
				item.toBeDeleted = true;
				this.removeItem(item);
				this.newItem(item.name, to);
				return;
			}
		} else {			
			this.moveItemInRootbyItem(item, to);				 	
			return;
		}
	},
	
	// dijit.tree.Model  API
	isItem: function() {
		return false;
	},

	// dijit.tree.Model  API
	getLabel: function(/*dojo.data.Item*/ item){
		if(item.type == NodeTypes.CATEGORY_VALUE_TYPE){
			var itemType = item.categoryType || item.ancestorType;
			var baseType = this.categoryBase[itemType.itemId || itemType.name];
			var baseValue = baseType.allValues[item.itemId || item.name];
			return baseValue.name;
		} else if(item.type == NodeTypes.CATEGORY_TYPE){
			var baseType = this.categoryBase[item.itemId || item.name];
			var requiredStar = baseType.required ? '<span style="color: red;"> *</span>' : "";
			return baseType.name + requiredStar;
		} else {
			var label = item.label?item.label:item.name;
			return label;
		}
	},

	// dijit.tree.Model  API
	mayHaveChildren: function(/*dojo.data.Item*/ item){
		if(item.type == NodeTypes.CATEGORY_TYPE){
			//by default return true
			return true;
		}else if(item.type == NodeTypes.CATEGORY_VALUE_TYPE){
			var parentTypeBase =  this.categoryBase[item.ancestorType.itemId || item.ancestorType.name];
			if(parentTypeBase.childTypes && parentTypeBase.childTypes.length > 0){
				return true;
			}
			item.isReady = true;
			return false;
		}else {
			//root
			return Boolean(item.children && item.children.length > 0 || item.childTypes && item.childTypes.length > 0);	
		}        
	},
	
	/*
	 * itemName:  New Item Name
	 * parentItem: parent container, null if root
	 * isSubcategory: If true, the new categoryType will be associated to either parentItem type, or if parentItem is a value, will be associated directly to value's parent type  
	 * 
	 * Returns: Items changed
	 * */		
	newItem: function(/*String*/ itemName , parentItem, isSubCategory){ // Creates a new item
		// Drag and Drop operations call newItem with an object instead of a name.
		// We need to inpect what comes and react accordingly
		if (typeof itemName === 'object') {
			var data = itemName;

			if (this.typeHierarchyOnly) {
				// Outline mode
				// Just call _newItem with the name, parent and true for isSubcategory... 
				// We're dropping it's a subcategory no matter what!
				return this._newItem(data.value || data.name, parentItem, true);
			}

			// ... otherwise, in full view mode...

			// Summary: Helper function to create a value... code block wrapped in a function
			//    in order to call it from more than one place without repeating code..
			var createValue = dojo.hitch(this, function() {
				//this is nto really the new Item function, but 
				//drag and drop controller does not really validates if value already exists, so we will do a small validation here				
				if (!this.valueExistsInParentType(parentItem, data.value || data.name)){
					var category = this.createCategoryValueDummyItem(data.value || data.name);
					return this._addNewCategoryValueToType(category, parentItem);
				}
				//Ignore otherwise, value already exists
			});

			if (data.isAssociationCandidate) {
				if(this.valueExistsInParentType(parentItem, data.value || data.name)){
					//value already exists in parentItem, no need to re-add it
					return;
				}				
				// Create an association - only if the parentItem is NOT a category type
				var categoryType = data.categoryType;
				categoryType = this.categoryBase[categoryType.itemId || categoryType.name];
				// Get the parent ite as the category type in categoryBase
				var parentItemAsCategoryType = this.categoryBase[parentItem.itemId || parentItem.name];
								
				// verify they're the same
				if (((categoryType.itemId !== undefined) && 
				 	 (categoryType.itemId === parentItemAsCategoryType.itemId))  || 
					 (categoryType.name === parentItemAsCategoryType.name)) {
					// The item being dropped is a child value of the same category type 
					// where the item is being dropped (all good.)
					var value = this._searchCategoryValueFromBase(categoryType, data.itemId || data.name);
					return this.associateExistingValueInParentSubType(parentItem, value);		
				} else {
					// The item is being dropped under a different category type... 
					// Create a value... even if we're in associations mode, in that case, the panel values
					// provides the item as a 'term', i.e. a dumb piece of text.
					return createValue();
				}		
			} else {
				// Create a value
				return createValue();
			}
		} else {
			// Natural newItem's code path...	
			if(!this.typeHierarchyOnly){
				return this._newItem(itemName, parentItem, isSubCategory);
			} else {
				return this._newItemHierarchyOnly(itemName, parentItem, isSubCategory);
			}
		}
	},
	
	removeItem: function(item, parentItem/*used only in hierarchy mode*/){
		if(!this.typeHierarchyOnly){
			return this._removeItem(item);
		} else {
			return this._removeItemHierarchyOnly(item, parentItem);
		}		
	}, 
	
	updateItem: function(item){		
		if(!this.typeHierarchyOnly){
			return this._updateItem(item);
		} else {
			return this._updateItemHierarchyOnly(item);
		}			
	},
	
	/**
	  intended to associate sub values to a parent value (NOT creating new, if you wish to create new items use NewItem)
	 **/
	associateExistingValueInParentSubType: function(parentItem, existingValue){
		return this._associateExistingValueInParentSubType(parentItem, existingValue);
	},
	
	/**
	  intended to associate sub types to a parent type (NOT creating new, if you wish to create new items use NewItem)
	 **/
	associateExistingTypeInParentType: function(parentItem, existingType, from){		
		return this._associateExistingTypeInParentType(parentItem, existingType, from);
	},
	
	getUnsavedCategoryValues: function() {
		var categoryTypes = this.getAllCategoryTypes();
		var allUnsavedValues = [];
		dojo.forEach(categoryTypes, function(categoryType) {
			var categoryValues = this.getAllCategoryValuesForType(categoryType.itemId || categoryType.name);
			var unsavedValues = dojo.filter(categoryValues, function(categoryValue) {
				return categoryValue.isNew;
			});
			allUnsavedValues = allUnsavedValues.concat(unsavedValues);
		}, this);
		return allUnsavedValues;
	}
	
});
})();
