/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2014. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.extension.QualityObjectivesSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.parser");
dojo.require("jazz.core.loader");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.client.internal.ObjectiveClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.construction.QualityObjectivesGrid");

dojo.require("dojox.grid.DataGrid");
dojo.require("dojo.data.ItemFileWriteStore");

(function() {
	var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
	
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var ObjectiveClient = com.ibm.rqm.planning.web.client.internal.ObjectiveClient;
	var PlanningPageClient = com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
	var GridHelper = com.ibm.asq.common.web.ui.internal.widgets.grid.GridContainerHelper;
	
	var COMMAND_CREATE = "create"; //$NON-NLS$
    var COMMAND_UPDATE = "update"; //$NON-NLS$
    var COMMAND_REMOVE = "remove"; //$NON-NLS$
    
	var STATE_SUCCESS = 	"com.ibm.rqm.planning.objectives.success"; //$NON-NLS$
	var STATE_EXCEPTION = 	"com.ibm.rqm.planning.objectives.success_exception"; //$NON-NLS$
	var STATE_FAILURE = 	"com.ibm.rqm.planning.objectives.unsuccessful"; //$NON-NLS$
	var STATE_NOT_STARTED = "com.ibm.rqm.planning.objectives.not_started"; //$NON-NLS$
	var STATE_UNRESOLVED = 	"com.ibm.rqm.planning.objectives.unresolved"; //$NON-NLS$
	
	var PERCENT_SEV2_DEFECTS_ID = 	"objectives.defects.open.sev2s"; //$NON-NLS$
	var NUMBER_SEV1_DEFECTS_ID = 	"objectives.defects.open.sev1s"; //$NON-NLS$
	// Enhancement 50733: For the moment, this enhancement will be limited to take into account only those Defects
	// which Severity is established as 1, 2 based on Out of the Box defaults for CCM and CQ.
	var CCM_DEF_SEV1_BLOCKER = "l6"; //$NON-NLS$ - 'Blocker'
	var CCM_DEF_SEV1_CRITICAL = "l5"; //$NON-NLS$ - 'Critical'
	var CCM_DEF_SEV2 = "l4"; //$NON-NLS$ - 'Major'
	var CQ_DEF_SEV1 = "1-Critical"; //$NON-NLS$
	var CQ_DEF_SEV2 = "2-Major"; //$NON-NLS$
	

    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.extension.QualityObjectivesSection", 
    		[dijit._Widget, dijit._Templated,
    		 com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection, 
    		 GridHelper], {
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/extension/QualityObjetivesSection.html"), 
        widgetsInTemplate:true,

        labelAddRowAction: Constants.getMessage("labelAddQualityObjective"),
        labelDeleteRowAction: Constants.getMessage("labelRemoveQualityObjective"),
        
        constructor: function() {
			this.labelObjective = Constants.getMessage("labelObjective");
			this.labelExpected =Constants.getMessage("labelExpected");
			this.labelActual =Constants.getMessage("labelActual");
			this.labelStatus= Constants.getMessage("labelStatus");
			this.labelComment= Constants.getMessage("labelComment");
			this.labelDescription =Constants.getMessage("labelDescription");
			this._states = [
                {id: STATE_SUCCESS, name: Constants.getMessage(STATE_SUCCESS)},
				{id: STATE_EXCEPTION, name: Constants.getMessage(STATE_EXCEPTION)},
				{id: STATE_FAILURE, name: Constants.getMessage(STATE_FAILURE)},
				{id: STATE_NOT_STARTED, name: Constants.getMessage(STATE_NOT_STARTED)},
				{id: STATE_UNRESOLVED, name: Constants.getMessage(STATE_UNRESOLVED)}
			];
			this.statusLabels= this._states[0].name + "," + this._states[1].name + "," + this._states[2].name + "," + this._states[3].name + "," + this._states[4].name ;
			this._actionButtons = {};
		},
        
        postCreate: function(){
            //we need to call AbstractArtifactEditorSection's postCreate, GridHelper does not have postCreate so we are ok. 
            this.inherited("postCreate", arguments);
            
            this._objStatuses = [];     
			this._evalEnabled = false;               
			this.addWorkItemSupport();

            // dojox.grid.DataGrid uses dojox.grid._Grid which uses dojox.html.metrics to track font resize
            // it creates and uses an iframe with no name and top = "-10000px" to be out of sight
            var iframes = dojo.query('> iframe', dojo.body());
            // get all immediate iframes in the body
            if (iframes.length > 0) {
                dojo.forEach(iframes, function(iframe){
                    // make sure we are hiding the iframe from the grid
                    if (!iframe.name && (iframe.style.top && iframe.style.top == "-10000px")) {
                        iframe.name = "hidden-grid-resize-frame";
                        dijit.setWaiState(iframe, "hidden", true);  // this will hide the frame from the screen reader
                    }
                });
            }
        },
        
        //will be lazily loaded by getMainContent
        //@Overridden from AbstractArtifactEditorSection    
        initialize: function(isEditable){
			this.inherited("initialize", arguments);
			this.refreshSection();
			var artifact = this.getArtifact();
			if (artifact != null) {
	            var args = {
	                planItemId: artifact.versionableItemId,
					groupId: this.getConfig().id
	            };
	            var respHandler = new ServiceResponseHandler(this, "_initialize", "_error");
	            ObjectiveClient.getObjectiveStatuses(respHandler, args);				
			} else {
				// the section has not yet ben open, try again in half a second
                // unless it is open
				setTimeout(dojo.hitch(this, "_initialize", null), 300);	
			}
        },
		
		_initialize: function(statuses) {
			
			this._objStatuses = statuses || [];
			                    
			this._initializeGrid();
            this._showGrid();
            this._setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage( this.getSection().name)));
            this._evalEnabled = this._hasEvalObjectives();
			this._addGridActions();
			this._updateActionStates();
		},
        
        _initializeGrid: function(){
        
           var dataAsGridModel = this._toGridModel(this._objStatuses);
			
           var model = {label:'', items:dataAsGridModel};
           var data = new dojo.data.ItemFileWriteStore({data: model});
           this._grid.startup();
           this._grid.setStore(data);        
           this._grid.model = dataAsGridModel;           
           this.rowCount = this.pageSize;
           
           dojo.connect(this._grid, "onRowClick", this, "_updateActionStates");
           dojo.connect(this._grid, "onKeyDown", this, "_updateActionStates");
           dojo.connect(this, "deleteSelectedRows", this, "_onRemoveRows");
           dojo.connect(this._grid, "onStartEdit", this, "_onGridActivation");
           dojo.connect(this._grid, "onApplyCellEdit", this, "_onGridChange");
           
       },
       
       _setTableSummary: function(message) {
			dojo.attr(this._grid.domNode, "summary", message);
       },
		
		onAdd:function(){
			 var objectivesView = new com.ibm.rqm.planning.web.ui.internal.view.construction.QualityObjectivesGrid({
                isDialog: true,
				dialogTitle: Constants.getMessage("labelSelectObjectives"),
                editable: false,
                excluded: this._getExcludedFromAdd(),
                okCallback: dojo.hitch(this, "_doAddExisting")
            });	
		},
		
		
		_doAddExisting: function(selected) {
			var newStatuses = [];
			for (var i = 0; i < (selected || []).length; i++) {
				var newObjStatus = {
					actualValue: 0,					
				    state: STATE_NOT_STARTED, 
					comment: "",
					objective: selected[i]
				};
				newStatuses.push(newObjStatus);
			}
			//add the new status objects to the grid
			if (newStatuses.length > 0) { 
				this.appendRows(this._toGridModel(newStatuses));
				this._evalEnabled = this._hasEvalObjectives();
				this._onGridChange();
			}
		},
		
		_doEvaluate: function() {
			var artifact = this.getArtifact();
			if (artifact != null) {
				var statuses = this._fromGridModel(false, true);
				if (statuses && statuses.length > 0) {
					// Enhancement 50733: Detecting Defect related Objectives
					var sev1ObjectiveIncluded = this._findByObjectiveId(statuses, NUMBER_SEV1_DEFECTS_ID);
					var sev2ObjectiveIncluded = this._findByObjectiveId(statuses, PERCENT_SEV2_DEFECTS_ID);
					var hasOSLCDefectObjectives = sev1ObjectiveIncluded || sev2ObjectiveIncluded;
					
					var respHandler = new ServiceResponseHandler(this, (hasOSLCDefectObjectives ? "_evaluateOSLCDefectObjectives" : "_handleEvalUpdates"), "_error");
					ObjectiveClient.evaluateObjectives(respHandler, artifact.versionableItemId, statuses);
				}
			}
		},
		
		_evaluateOSLCDefectObjectives: function(evaluatedStatuses) {
				var artifact = this.getArtifact();
				
				this.alreadyEvaluated = evaluatedStatuses;
				// Get the list of Defects related to this Test Plan
				jazz.core.loader.batch_load_async(["com.ibm.rqm.defects.web.ui.view.common.ProgressDialog", "com.ibm.rqm.defects.web.client.internal.ResourceLinkService"], dojo.hitch(this, function(){
					com.ibm.rqm.defects.web.ui.view.common.ProgressDialog.doRunWithProgress("", dojo.hitch(this, function(monitor){
						monitor.beginTask(Constants.getMessage("labelEvaluateObjectives"));
						this.monitor = monitor;
					// Take into account Children Test Plans in case the plan is Roll Up Objectives
					if(artifact && artifact.isRollUpObjectives === true) {
						var respHandler = new ServiceResponseHandler(this, "_evaluateObjectives", "_error");
						PlanningPageClient.getTestPlanChildren(respHandler, {
							uid: artifact.versionableItemId,
							minimal: true
						});
					} else {
						this._evaluateObjectives(artifact.versionableItemId);
					}
				}));
			}));
		},
		
		_evaluateObjectives: function(scope) {
			var statuses = this._fromGridModel(false, true);
			var sev1ObjectiveIncluded = this._findByObjectiveId(statuses, NUMBER_SEV1_DEFECTS_ID);
			var sev2ObjectiveIncluded = this._findByObjectiveId(statuses, PERCENT_SEV2_DEFECTS_ID);
			
			// Include master test plan id in scope
			if(dojo.isArray(scope))
				scope.push({versionableItemId: this.getArtifact().versionableItemId});
			
			com.ibm.rqm.defects.web.client.internal.ResourceLinkService.getTestPlanDefectsRelated(this.getParentEditor().getArtifactType(), 
				scope, dojo.hitch(this, function(links) {
					// Evaluate the Defect related Objectives
					var newStatuses = this._evaluateDefectRelatedObjectives(links, this.alreadyEvaluated, sev1ObjectiveIncluded, sev2ObjectiveIncluded);
					this.monitor.done();
					this._handleEvalUpdates(newStatuses);
				}), dojo.hitch(this, function(result) {
					this.monitor.done();
					console.debug("Error aquiring Test Plan's Defect list: " + result);
			}));
		},
		
		// Evaluate Out of the Box Defect Based Objectives for OSLC integration providers
		_evaluateDefectRelatedObjectives: function(links, evaluatedStatuses, sev1ObjectiveIncluded, sev2ObjectiveIncluded) {
			var sev1Types = [CCM_DEF_SEV1_BLOCKER, CCM_DEF_SEV1_CRITICAL, CQ_DEF_SEV1];
			var sev2Types = [CCM_DEF_SEV2, CQ_DEF_SEV2];
			
			var countSev1 = 0;
			var countSev2 = 0;
			var countedURILinks = [];
			for(var i=0;links[i];i++) {
				var link = links[i];
				// Consider all defects that are not closed, have the severity required AND have not already been taken into account
				if(!link.closed && dojo.indexOf(sev1Types, link.severity) >= 0 && dojo.indexOf(countedURILinks, link.uri) < 0) {
					countSev1++;
					countedURILinks.push(link.uri);
				}
				if(!link.closed && dojo.indexOf(sev2Types, link.severity) >= 0 && dojo.indexOf(countedURILinks, link.uri) < 0) {
					countSev2++;
					countedURILinks.push(link.uri);
				}
			}
			
			if(sev1ObjectiveIncluded) {
				var newValue;
				var evaluated = evaluatedStatuses[sev1ObjectiveIncluded.groupOrder];
				var localValue = evaluated.actualValue;
				newValue = evaluated.actualValue = countSev1;
				var newStatus = this._evaluateCondition(evaluated.objective.operator, newValue, evaluated.objective.targetValue);
				evaluated.state = newStatus;
				
			}
			
			if(sev2ObjectiveIncluded) {
				var newValue;
				var evaluated = evaluatedStatuses[sev2ObjectiveIncluded.groupOrder];
				var localValue = evaluated.actualValue;
				newValue = countSev2;
				
				newValue = links.length == 0 ? 0 : dojo.number.round((newValue / links.length) * 100);
				evaluated.actualValue = newValue;
				var newStatus = this._evaluateCondition(evaluated.objective.operator, newValue, evaluated.objective.targetValue);
				evaluated.state = newStatus;
			}
			return evaluatedStatuses;
		},
		
		// JS version of server side evaluator function used with OSLC Defect related objectives
		_evaluateCondition: function(operator, actualValue, targetValue) {
			if(operator != null) {
				switch(operator) {
					case "=":
						return (actualValue == targetValue) ? STATE_SUCCESS : STATE_FAILURE;
					case ">":
						return (actualValue > targetValue) ? STATE_SUCCESS : STATE_FAILURE;
					case ">=":
						return (actualValue >= targetValue) ? STATE_SUCCESS : STATE_FAILURE;					
					case "<":
						return (actualValue < targetValue) ? STATE_SUCCESS : STATE_FAILURE;				
					case "<=":
						return (actualValue <= targetValue) ? STATE_SUCCESS : STATE_FAILURE;
					default:
						break;
				}
			}	
			return STATE_UNRESOLVED;
		},
		
		_handleEvalUpdates: function(statuses) {			
			statuses = statuses || [];
			var statusLen = statuses.length;
			var gridRows = this._fromGridModel(false);
			var grid = this._grid;
			var changesMade = false;
			for (var i = 0; i < statusLen; i++) {
			    var rowChanged = false;
				//update the grid rows with new evaluations, if they've changed
				var updated = statuses[i];
				var gridIndex = this._findGridRowIndex(gridRows, updated);
				var original = gridRows[gridIndex];
				var newActual = updated.actualValue;
				if (original.actualValue != newActual) {
				    rowChanged = true;
					grid.doApplyCellEdit(newActual, gridIndex, "actualValue");
				}
				var newState = updated.state;
				if (original.state != newState) {
				    rowChanged = true;
					grid.doApplyCellEdit(this._getStateName(newState), gridIndex, "state");
				}
				if (rowChanged) {
					grid.doApplyEdit(gridIndex);
					this._onGridChange();
					}
					changesMade = rowChanged ? true : changesMade;                
			}			
			 if(!changesMade) {
                //show info dialog indicating everything is up-to-date
                var args = {
                    primaryTitle: Constants.getMessage("labelEvaluateObjectives"),
                    message: Constants.getMessage("msgEvaluateNoChanges")
                };
                var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog(args);         
                alertDialog.setSecondaryTitle("");
                alertDialog.open();
               }
		},
		
		_findGridRowIndex: function(statuses, status) {
			var statusLen = (statuses || []).length;
			for (var i = 0; i < statusLen; i++) {
				if (statuses[i].groupOrder == status.groupOrder) return i;
			}						
			return -1;
		},
		
		_hasEvalObjectives: function() {
			//determine whether or not this section contains any objectives that can
			//be automatically evaluated by the system
			var statuses = this._fromGridModel();
			var objLen = (statuses || []).length;
			for (var i = 0; i < objLen; i++) {
				var evaluatorId = statuses[i].evaluatorId;
				if (evaluatorId != null && dojo.trim(evaluatorId).length > 0) {
					return true;
				}
			}
			return false;
		},
		
		_onGridSelection: function() {
			this._updateActionStates();
		},
		
		_onRemoveRows: function() {
			this._evalEnabled = this._hasEvalObjectives();
			this._onGridChange();
		},
		
        _onGridActivation: function() {
			this._updateActionStates();
        },		
        
        _onGridChange: function(inValue, inRowIndex, inAttrName) {
        	var oldValue;
        	if(inRowIndex && this._grid._cache[inRowIndex]){
        		oldValue = this._grid._cache[inRowIndex];
        	}
        	if(!oldValue || oldValue != inValue){
        		this.setDirty();
        	}
        },
		
		_updateActionStates: function() {
			if (this._grid && this._gridEvalButton) {
				this._gridEvalButton.set('disabled', !this._evalEnabled);
			}		
			var selected = this._grid.selection.getSelected();
		    this.setButtonDisabled(this.ACTION_REMOVE, !(selected != null && selected.length > 0));	
		},
		
		setButtonDisabled: function(id, isDisabled) {
        	var button = this._actionButtons[id];
        	if (button) {
            	button.set('disabled', isDisabled);
        	}
		},	
		
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.addWorkItemSupport();
			//remove temp commands on the editor
			if (this.getParentEditor()._tempQualObjCommands) {
				delete this.getParentEditor()._tempQualObjCommands;
			}	
			if (this.initialized) {
				this.initialize();
			} 
			else {
				// Restore view look & feel on not selected Objectives section, if any
				if(this._grid){
					this._grid.edit.apply();
				}
			}
        },
        
        onActivated: function(){
        	if(this._grid){
        		if(this._grid.edit){
    				this._grid.edit.apply();       				
        		}
        		this._showGrid();
        	}
        	this.inherited("onActivated", arguments);
        },
        
       
		
		//Override
		validate: function() {
            if (this.initialized) {
            	this._grid.edit.apply();
				var updates = this._fromGridModel();
				for (var i = 0; i < (updates || []).length; i++) {
					var actualValue = updates[i].actualValue;
		            if (actualValue === "") {
						var msg = Constants.getMessage("msgRequiredGridField", Constants.getMessage("labelActual"), (i + 1));
		                this.getParentEditor().setHeaderStatus(msg, Constants.HEADER_STATUS_ERROR, msg);
						return false;
					} else if (this._isNonNumeric(actualValue)) {
						var msg = Constants.getMessage("msgInvalidGridField", Constants.getMessage("labelActual"), Constants.getMessage("msgValidNumber"), (i + 1));
						this.getParentEditor().setHeaderStatus(msg, Constants.HEADER_STATUS_ERROR, msg);
		                return false;
		            }
				}
			}
			return true;
		},
		
		_isNonNumeric: function(value) {
			var asNum = new Number(value);	
			if(isNaN(asNum) || asNum != parseInt(asNum)){
				return true;
			}
			return false;
		},		
		
		appendRows: function(rows) {
			for (var i = 0; i < (rows || []).length; i++) {
				this.appendRow(rows[i]);
			}
		},
	
		//@Overriden from AbstractArtifactEditorSection
        getAttributes: function(){
            if (this.initialized) {
				var commands = this.getParentEditor()._tempQualObjCommands || [];
				var updates = this._fromGridModel();
				var isCopy = this.getParentEditor().isCopy();
				var originals = this._objStatuses;
				//handle creates & updates 				
				for (var i = 0; i < (updates || []).length; i++) {
					var u = updates[i];
					if (u.id != null) {
						var o = this._findIn(originals, u);
						if (o != null) {
							if (o.comment == u.comment && o.state == u.state && o.actualValue == u.actualValue) 
								continue; //no change
						}
					}
					commands.push({
						cmd: u.id != null && !isCopy ? COMMAND_UPDATE : COMMAND_CREATE,
						statusObj: u
					});
				}
				//handle removals
				for (var i = 0; i < (originals || []).length; i++) {
					var o = originals[i];
					if (!this._findIn(updates,o)) {
	                    commands.push({
	                        cmd: COMMAND_REMOVE,
	                        statusObj: {id: o.id}
	                    });		
					}
				}
				//temporarily store commands on the editor
				this.getParentEditor()._tempQualObjCommands = commands;
				
                return {
                    updateObjectivesJson: dojo.toJson(commands)
                }               
            }
            return null;
        },		
		
        //@Overriden from AbstractArtifactEditorSection
        isSummaryEditable: function(isEditable){
            return isEditable;
        },
        
        //@Overriden from AbstractArtifactEditordescription
        getSummaryChangeHandler: /* Function */ function(){
            return dojo.hitch(this, "_summaryChanged");
        },		
		
        _findIn: function(statuses, status) {
            for (var i = 0; i < (statuses || []).length; i++) {
                if (statuses[i].id != null && statuses[i].id == status.id) {
                    return statuses[i];
                }
            }
            return null;
        },
        
        _findByObjectiveId: function(statuses, objectiveId) {
            for (var i = 0; i < (statuses || []).length; i++) {
                if (statuses[i].objectiveId != null && statuses[i].objectiveId == objectiveId) {
                    return statuses[i];
                }
            }
            return null;
        },
		
		_getStateNames: function() {
			var namesOnly = [];
			for (var i = 0; i < this._states.length; i++) {
				namesOnly.push(this._states[i].name);
			}
			return namesOnly;
		},

		_getStateName: function(id) {
			var state = this._getState("id", id); //$NON-NLS$
			return (state != null ? state.name : null);
		},
				
		_getStateId: function(name) {
			var state = this._getState("name", name);  //$NON-NLS$
			return (state != null ? state.id : null);
		},
				
		_getState: function(property, value) {
			for (var i = 0; i < this._states.length; i++) {
				if (this._states[i][property] == value) {
					return this._states[i];
				}
			}
			return null;
		},	
        
        _summaryChanged: function(oldValue, newValue){
            this.description = newValue;
        },
        
        _showGrid: function(){
            if (this._grid) {
                this._grid.render();
            }
        },
		
		_getExcludedFromAdd: function() {
            var statuses = this._objStatuses || [];
			statuses = statuses.concat(this._fromGridModel());
			var excluded = {};
            for (var i = 0; i < (statuses || []).length; i++) {
                var s = statuses[i];
                if (s.objective != null) {
					excluded[s.objective.itemId] = true;
				} else if (s.objectiveItemId != null) {
					excluded[s.objectiveItemId] = true;
				}
            }
            return excluded;			
		},
		
        _fromGridModel: function(selectedOnly, includeObjectiveId) {
            var asModelled = [];
            if (this._grid) {
                //each row is an array of values
                var rows = selectedOnly === true ? this._grid.selection.getSelected() : this._grid.store._arrayOfAllItems;
                for (var i = 0; i < rows.length; i++) {
                	var row = rows[i];
                	if(row){
						var modelledRow = 
                    	{
                        	name: dojo.isArray(row.name) ? row.name[0]: (row.name || ""), 
                        	expected: dojo.isArray(row.targetOperator) ? row.targetOperator[0]: (row.targetOperator || ""),
                        	actualValue: dojo.isArray(row.actualValue) ? (row.actualValue[0]>=0? row.actualValue[0] : ""): ((row.actualValue>=0 ? row.actualValue : "") || ""),
                        	state: dojo.isArray(row.state) ? this._getStateId(row.state[0]): (this._getStateId(row.state) || ""), 
                        	comment: dojo.isArray(row.comment) ? row.comment[0]: (row.comment || null),
                        	objectiveItemId: dojo.isArray(row.itemID) ? row.itemID[0]: (row.itemID || null),
							objectiveStateId: dojo.isArray(row.stateId) ? row.stateId[0]: (row.stateId || null),
							group: this.getSection().id,
							groupOrder: i
                    	};
						if(row.id){
							var id = dojo.isArray(row.id) ? row.id[0]: row.id;
							if(id != null)
								modelledRow.id = id;
						}
						if(row.evaluatorId){
							var evaluatorId = dojo.isArray(row.evaluatorId) ? row.evaluatorId[0]: row.evaluatorId;
							if(evaluatorId != null)
								modelledRow.evaluatorId = evaluatorId;
						}
						// Include the Objective Id to tell if a Defect Quality Objective is included in evaluation
						if(includeObjectiveId){
							var objectiveId = dojo.isArray(row.objectiveId) ? row.objectiveId[0]: row.objectiveId;
							if(objectiveId != null)
								modelledRow.objectiveId = objectiveId;
						}
						asModelled.push(modelledRow);
                	}	
                }
            }
            return asModelled;
        },
        
        _toGridModel: function(statuses) {
            var asGridModel = [];
            for (var i = 0; i < (statuses || []).length; i++) {
                var s = statuses[i];
				var o = s.objective;
				var target = o.targetValue != "" ? new String(o.targetValue) : "0";
                asGridModel.push(
                	   {name:o.name, 
                    	targetOperator:(o.operator + " " + target),
                    	actualValue:s.actualValue, 
                    	state:this._getStateName(s.state),
                    	description: o.description,
                    	comment:s.comment||"", 
                    	itemID: o.itemId, 
                    	stateId: o.stateId, 
                    	id: (s.id != null ? s.id : null ), 
                    	evaluatorId:o.evaluatorId?o.evaluatorId:null,
                    	objectiveId:o.id?o.id:null
                    });
            }
            return asGridModel;     
        },
        
        _addGridActions: function() {
		    if (this._grid) {
				//add the evaluate action
				if(!this._gridEvalButton){
					var evaluateAction = {
						id: "evaluate", //$NON-NLS$
						label: Constants.getMessage("labelEvaluateObjectives"), //$NON-NLS$
						callback: dojo.hitch(this, "_doEvaluate"),
						iconClass: "evaluate-icon-image", //$NON-NLS$
						disabled: false
					}				
					this._gridEvalButton = this.addAction(evaluateAction);
				}
				
				//add the 'manage' action if the section is editable
				if (this.isEditable() && !this._gridManageButton) {
					var permission = PermissionsClient.getStoredPermission(PermissionsClient.OP_TEST_OBJECTIVE);
					if (permission && (permission.create || permission.edit)) {
						var manageAction = {
							id: "manage", //$NON-NLS$
							label: Constants.getMessage("labelManageObjectives"), //$NON-NLS$
							callback: dojo.hitch(this, function(e){
								var objectivesView = new com.ibm.rqm.planning.web.ui.internal.view.construction.QualityObjectivesGrid({
									isDialog: true,
									editable: true
								});
							}),
							iconClass: "objectives-icon-image",
							disabled: false
						}
					this._gridManageButton = this.addAction(manageAction);
					}
				}
				if (this.isEditable())
						this._addDefaultActions();
			}
		},
        
		
        _error: function(e){
            console.error(this.declaredClass + ": An error occurred: \n[" + e.message + "]");
        }		
    });
})();
