/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.ExplorableTableFilterProvider");

// An implementation of TableContentFilterProvider that is used in the explorer view.
//
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.client.internal.FiltersQueriesClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.SaveQueryDialog");

(function() {
    
var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var FiltersQueriesClient = com.ibm.asq.common.web.client.internal.FiltersQueriesClient;
var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
var CommonUtility = com.ibm.asq.common.web.ui.internal.util.Utility;

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.ExplorableTableFilterProvider", 
    com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentFilterProvider,
{
    allFiltersQueries: null,  // will be non-null when the saved queries are initialized
    loadingQueries: false,    // track of async loading of the saved queries
    saveQueriesSupport: true, // set to false to not use saved queries
    projectAreaItemId: null,
    noPersonalQueries : false,
    delayQueriesInitialization: false, 		// At least for keywords, the code originally depended on initializing the filterprovider early on.
                                			// To support the composite web service, I added the queryFilters args, but I could not use that here.
                            				// The issue was with just that choice, this code would initialize the filters if they were not passed in.
    										// This lets me delay this initialization, and cvall setFilters later.  
    
    constructor: function(args) {
        dojo.mixin(this, args);  
        
        // initialize the saved queries that will be used in the explorer
        if (!this.artifactTypeQueryKey) {
            var error = new Error("artifactTypeQueryKey is not specified"); // $NON-NLS$ dev usage error
            console.error(error);
        } else if (!this.delayQueriesInitialization && this.saveQueriesSupport) {
        
        	if (args && args.queryFilters) {
        		this.setFilters(args.queryFilters);
            } else {
            	this.getAllSavedQueries(null, null);
            }
        }
    },

    getAllSavedQueries: function(successCallback, errorCallback, refresh) {
        if (this.allFiltersQueries && !refresh && successCallback) {
            // already fetched the queries before, use cache unless force to refresh
            successCallback(this.allFiltersQueries);
        } else if (this.loadingQueries) {
            // queries is loading, user may be expanding the explorer before loading is completed
            // save the handlers to invoke upon loading complete
            this.asyncLoadListener = {
                success: successCallback,
                error: errorCallback
            };
        } else {
            // set loading flag 
            this.loadingQueries = true;
            var self = this;
            var _gotFilters = function(response) {
            		if(self.noPersonalQueries && response){
            			response = self._filterSavedQueries(response);
            		}
            		
                	self.setFilters(response);
                    self.loadingQueries = false;
                    if (successCallback) {
                        successCallback(self.allFiltersQueries);
                    }
                    if (self.asyncLoadListener && self.asyncLoadListener.success) {
                        self.asyncLoadListener.success(self.allFiltersQueries);
                        delete self.asyncLoadListener;
                    }
            };
            if (this._compositeDTO) {
            	_gotFilters(this._compositeDTO.queryFilters);
            } else {
	            var handler = new ServiceResponseHandler({
	                success: function(response){
	                	_gotFilters(response);
                },
                error: function(e){
                    self.loadingQueries = false;
                    if (errorCallback) {
                        errorCallback(e);
                    } else {
                        console.error(e);
                    }
                    if (self.asyncLoadListener && self.asyncLoadListener.error) {
                        self.asyncLoadListener.error(e);
                        delete self.asyncLoadListener;
                    }
                }
            },
            "success",
            "error");
            FiltersQueriesClient.getQueries(handler, this.artifactTypeQueryKey, this.projectAreaItemId);
        }
        }
    },
    
    setFilters: function(response) {
        this._createDefaultScopes();
        if (dojo.isArray(response) && response.length > 0) {
    	var self = this;
            dojo.forEach(response, function(query) {
                var filters = FiltersQueriesClient._getFilters(query.values);
                var savedQuery = {
                    "type": query.type,
                    "itemId": query.itemId,
                    "stateId": query.stateId,
                    "folder": query.folder,
                    "title": query.name,
                    "inMenu": query.menu,
                    "scope": query.scope,
                    "filters": filters
                };
                // save query in organized cache
                self._organizeQuery(savedQuery);
            });
        }

    },
    
    _createDefaultScopes: function() {
        this.allFiltersQueries = {};
        if(!this.noPersonalQueries){
        	this.allFiltersQueries[FiltersQueriesClient.privateScopeId] = {
                    itemId: FiltersQueriesClient.privateScopeId,
                    label: Constants.getCommonMessage("labelMyQueries"),
                    folders: {},
                    queries: {}
                };
        }
        
        this.allFiltersQueries[FiltersQueriesClient.publicScopeId] = {
            itemId: FiltersQueriesClient.publicScopeId,
            label: Constants.getCommonMessage("labelSharedQueries"),
            folders: {},
            queries: {}
        };
    },
    
     _filterSavedQueries: function(response){
    	 return dojo.filter(response, function(item) {
             return item.scope != "USER";
         });    	
    },
    
    // append scope to folder name to make unique itemId for tree model
    getModelFolderId: function(folder, scope) {
        return folder + "." + scope;
    },
    
    // Queries are a flat list but are cached in associative arrays based on their scope and
    // folder into.  The member variable this.allFiltersQueries is an associative array of two scopes
    // public (PROJECT_AREA) and private (USER).  The children of the two scopes are associative
    // array of folders and/or queries.  The children of the folders are associative array of queries.
    //
    _organizeQuery: function(query) {
    
        var scope = query.scope;
        var scopeLabel = scope;
        // we only have two predefined public and private scopes for now, this is for future reference
        if (!this.allFiltersQueries[scope]) {
            // create scope that is not created yet
            this.allFiltersQueries[scope] = {
                itemId: scope,
                label: scopeLabel,
                folders: {},
                queries: {}
            };
        }
        // default to save the query without folder i.e. top level of the scope
        var queriesNode = this.allFiltersQueries[scope].queries;
        var folder = query.folder;
        if (folder) {
            // query is within a folder
            var queryFolder = this.allFiltersQueries[scope].folders;
            if (!queryFolder[folder]) {
                // folder itemId is only used by the tree model
                var folderId = this.getModelFolderId(folder, scope);
                // create folder within scope that is not created yet
                queryFolder[folder] = {
                    itemId: folderId,
                    label: folder,
                    queries: {}
                    // note that folder does not have children folder
                };
            }
            // update to save the query within the folder of the scope
            queriesNode = queryFolder[folder].queries;
        }
        queriesNode[query.title] = {
            itemId: query.itemId,
            label: query.title,
            query: query
        };
    },
    
    _deleteOrganizedQuery: function(query, keepFolder) {
        var scope = this.allFiltersQueries[query.scope];
        var folder = query.folder ? scope.folders[query.folder] : null;
        var title = query.title;
        if (folder) {
            // delete query within a folder
            delete folder.queries[title];
            // delete the folder as well if is empty unless it needs to be kept for edit query
            if (!keepFolder) {
                var emptyFolder = true;
                for (var key in folder.queries) {
                    emptyFolder = false;
                    break;
                }
                if (emptyFolder) {
                    delete scope.folders[query.folder];
                }
            }
        }
        else {
            // delete query not in a folder
            delete scope.queries[title];
        }
    },
    
    _findQueryByTitle: function(container, title) {
        return container.queries[title];
    },
    
    _findQueryByItemId: function(container, itemId) {
        for (var title in container.queries) {
            var query = container.queries[title];
            if (query.itemId == itemId) {
                return query;
            }
        }
        return null;
    },
    
    findQuery: function(queryObj) {
        var scope = queryObj.scope || queryObj.sqScope;
        var folder = queryObj.folder || queryObj.sqFolder;
        var title = queryObj.title || queryObj.sqTitle;
        var itemId = queryObj.itemId || queryObj.sqItemId;
        var query = null;
        if (scope && this.allFiltersQueries[scope] && (title || itemId)) {
            if (folder) {
                var containerFolder = this.allFiltersQueries[scope].folders[folder];
                if (containerFolder) {
                    if (itemId) {
                        query = this._findQueryByItemId(containerFolder, itemId);
                    } else {
                        query = this._findQueryByTitle(containerFolder, title);
                    }
                }
            } else {
                var containerScope = this.allFiltersQueries[scope];
                if (itemId) {
                    query = this._findQueryByItemId(containerScope, itemId);
                } else {
                    query = this._findQueryByTitle(containerScope, title);
                }
            }
        }
        return query;
    },
    
    onQuerySaved: function(parentItem, queryItem, savedQuery, originalQueryObj) {
        // listener to invoke upon a query is saved
        if (this.tableView) {
            if (dojo.isFunction(this.tableView.onSavedQuerySaved)) {
                this.tableView.onSavedQuerySaved(savedQuery);
            }
        }
    },
    
    onError: function(error) {
        // error handler to display error condition
        if (this.tableView && dojo.isFunction(this.tableView.onError)) {
            this.tableView.onError(error);
        }
    },

    _createQueryIconClassLookup: function() {
        // compose the icon class lookup array for various artifact types
        this.iconClassSprites = {};
        this.iconClassSprites[FiltersQueriesClient.REQUIREMENT_QUERY_KEY] = "saved-queries-icon-requirement";
        this.iconClassSprites[FiltersQueriesClient.TEST_PLAN_QUERY_KEY] = "saved-queries-icon-test-plan";
        this.iconClassSprites[FiltersQueriesClient.TEST_CASE_QUERY_KEY] = "saved-queries-icon-test-case";
        this.iconClassSprites[FiltersQueriesClient.TEST_SCRIPT_QUERY_KEY] = "saved-queries-icon-test-script";
        this.iconClassSprites[FiltersQueriesClient.TEST_SUITE_QUERY_KEY] = "saved-queries-icon-test-suite";
        this.iconClassSprites[FiltersQueriesClient.TEST_DATA_QUERY_KEY] = "saved-queries-icon-test-cell";
        this.iconClassSprites[FiltersQueriesClient.KEYWORDS_QUERY_KEY] = "saved-queries-icon-keyword";
        this.iconClassSprites[FiltersQueriesClient.BUILD_REC_QUERY_KEY] = "saved-queries-icon-build-record";
        this.iconClassSprites[FiltersQueriesClient.BUILD_DEF_QUERY_KEY] = "saved-queries-icon-build-definition";
        this.iconClassSprites[FiltersQueriesClient.TER_QUERY_KEY] = "saved-queries-icon-case-exe-record";
        this.iconClassSprites[FiltersQueriesClient.SER_QUERY_KEY] = "saved-queries-icon-suite-exe-record";
        this.iconClassSprites[FiltersQueriesClient.TER_TC_RESULT_QUERY_KEY] = "saved-queries-icon-case-exe-result";
        this.iconClassSprites[FiltersQueriesClient.TER_TS_RESULT_QUERY_KEY] = "saved-queries-icon-suite-exe-result";
        this.iconClassSprites[FiltersQueriesClient.EXE_SCHEDULE_QUERY_KEY] = "saved-queries-icon-exe-schedule";
        this.iconClassSprites[FiltersQueriesClient.EXE_CONSOLE_QUERY_KEY] = "saved-queries-icon-query";
        this.iconClassSprites[FiltersQueriesClient.ADAPTER_CONSOLE_QUERY_KEY] = "saved-queries-icon-query";
        this.iconClassSprites[FiltersQueriesClient.PRINT_JOB_QUERY_KEY] = "saved-queries-icon-query";
    },

    getQueryIconClass: function(query) {
        if (!this.iconClassSprites) {
            this._createQueryIconClassLookup();
        }
        // default icon is the plain query icon
        var iconClass = this.iconClassSprites[this.artifactTypeQueryKey] || "saved-queries-icon-query";
        
        if (query && query.inMenu) {
            // query is promoted to be in the top level app menu, use different icon
            iconClass = "saved-queries-icon-in-menu";
        }
        return iconClass;
    },
    
    saveQuery: function() {
        var successCallback = dojo.hitch(this, "onQuerySaved");
        var errorCallback = dojo.hitch(this, "onError");
        var dialogParams = {
            filterProvider: this,
            allQueries: this.allFiltersQueries,
            csHelpID: "rqm_filters_query_save.html",
            okCallback: dojo.hitch(this, function(queryParams) {
                if (this.phantomSelectedQuery) {
                    // clear the phantom selected query will also clear the message box
                    this.setPhantomSelectedQuery(null);
                } else {
                    // clear filter massage box
                    this.clearMessageBox();
                }
                this.onSaveQuery(/* parent item */null, /* query item */null, queryParams, /* original query */null, successCallback, errorCallback);
            })
        };
        
        // check if we are currently showing a saved query
        var contentArea = dojo.getObject("com-ibm-rqm-planning-web-page-content-area");
        var tabsContainer = contentArea ? contentArea._tabs : null;
        var currentPage = tabsContainer ? tabsContainer.lastSelectedPage : null;
        
        if (currentPage) {
            var uriParams = CommonUtility.getArgsFromUri(currentPage.unfilteredUri);
            if (this.phantomSelectedQuery) {
                // use params from the phantom query if we are showing one, the uri might not have all the params
                uriParams = this.phantomSelectedQuery;
            }
            if (uriParams.sqScope) {
                dialogParams.initScope = uriParams.sqScope;
            }
            if (uriParams.sqFolder) {
                dialogParams.initFolderName = decodeURIComponent(uriParams.sqFolder);
            }
            if (uriParams.sqTitle) {
                dialogParams.initQueryName = decodeURIComponent(uriParams.sqTitle);
            }
        }
        var saveDialog = new com.ibm.rqm.planning.web.ui.internal.view.common.SaveQueryDialog(dialogParams);
        saveDialog.show();
    },
    
    editQuery: function(parentItem, queryItem, successCallback, errorCallback) {
        var query = queryItem.query;
        var saveDialog = new com.ibm.rqm.planning.web.ui.internal.view.common.SaveQueryDialog({
            initScope: query.scope,
            initFolderName: query.folder,
            initQueryName: query.title,
            edit: true,
            filterProvider: this,
            allQueries: this.allFiltersQueries,
            csHelpID: "rqm_filters_query_edit.html",
            okCallback: dojo.hitch(this, function(queryParams) {
                this._saveQuery(parentItem, queryItem, queryParams, null, query, successCallback, errorCallback);
            })
        });
        saveDialog.show();
    },
    
    onSaveQuery: function(parentItem, queryItem, queryValues, originalQueryObj, successCallback, errorCallback) {
        if (queryValues.title) {
            var viewer = this.tableView.tableViewer;
            var contentProvider = viewer.contentProvider;
            var headerRenderer = viewer.headerRenderer;
            var filters = headerRenderer.getSelectedFilters();
            
            if(this.tableView && this.tableView.processFilterForSave){
            	this.tableView.processFilterForSave(filters);
            }

            // auto-save can occur when no filters have been set
            if (filters == null) {
                filters = {};
            }
            
            // add sort info to filters
            var sortArgs = null;
            if (viewer.allowMultiSort) {
                sortArgs = contentProvider.buildGroupSortServerArgs();
            } else {
                // create the same sort format as multi-sort
                var sortCol = viewer.getSortByColumn();
                var sortDir = viewer.getSortDirection();
                if (sortCol > -1) {
                    sortCol = contentProvider.getColumnIdName(sortCol);
                    // map the table viewer sort direction to query sort direction
                    sortDir = (sortDir == TableConstants.SORT_DIR_ASCEND) ? TableConstants.QUERY_SORT_DIR_ASCEND : TableConstants.QUERY_SORT_DIR_DESCEND;
                    sortArgs = [ sortCol + ":" + sortDir ];
                }
            }
            if (sortArgs && sortArgs.length > 0) {
                filters['sortColumns'] = sortArgs;
            }
            
            // add group by info to filters
            var groupBy = viewer.getGroupByColumn();
            if (groupBy > -1) {
                // save the group by info by column id
                filters[TableConstants.QUERY_GROUP_BY_KEY] = contentProvider.getColumnIdName(groupBy);
            }
            this._saveQuery(parentItem, queryItem, queryValues, filters, originalQueryObj, successCallback, errorCallback);
        }
    },

    // The server call to save or edit a query.  Edit will only update the scope, folder and title of a query.  Save
    // will be an update if the query already exists and only the filters will be updated.
    //
    // Params: 
    //   parentItem - the parent item of the query tree item, null for save query
    //   queryItem - the query tree item to be edited, maybe for save query
    //   queryObj - scope, folder and title params to be saved or edited
    //   filters - null for edit query
    //   originalQueryObj - the original query obj to be edited, null for save query
    //   successCallback - callback for the server call on success
    //   errorCallback - callback for the server call on error
    //
    _saveQuery: function(parentItem, queryItem, queryObj, filters, originalQueryObj, successCallback, errorCallback){
        var nonNullFilters = {};
        if (originalQueryObj) {
            // keep existing filters for edit query
            nonNullFilters = originalQueryObj.filters;
        } else {
            // remove null value filters
            for (var key in filters) {
                var value = filters[key];
                if (value === null || value === "" || (dojo.isArray(value) && value.length == 0)) {
                    continue;
                }
                nonNullFilters[key] = value;
            }
        }
        
        // set up params
        var queryParam = [{
            "artifactType": this.artifactTypeQueryKey,
            "name": queryObj.title,
            "scope": queryObj.scope,
            "folder": queryObj.folder,
            "filters": [nonNullFilters]
        }];
        
        // check if query already exists for update instead of saving new, save will overwrite existing query
        if (queryObj.scope != FiltersQueriesClient.autoSaveScopeId) {
	        var updateQueryObj = originalQueryObj ? originalQueryObj : queryObj;
	        var updateQuery = this.findQuery(updateQueryObj);
	        
	        // update is on an existing filter, copy the other params
	        if (updateQuery) {
	            var updateQueryObj = updateQuery.query;
	            queryParam[0].itemId = updateQueryObj.itemId;
	            queryParam[0].stateId = updateQueryObj.stateId;
	            queryParam[0].isMenu = updateQueryObj.inMenu ? "true" : "false";
	        }
	    }
        
        var self = this;
        var handler = new ServiceResponseHandler({
            success: function(response){
                if (dojo.isArray(response) && response.length > 0) {
                    // expect only one response object
                    var query = response[0];
                    var filters = FiltersQueriesClient._getFilters(query.values);
                    var savedQuery = {
                        "type": query.type,
                        "itemId": query.itemId,
                        "stateId": query.stateId,
                        "title": query.name,
                        "folder": query.folder,
                        "inMenu": query.menu,
                        "scope": query.scope,
                        "filters": filters
                    };
                    // query saved successfully. If the user wants to retain the same columns
                    // for the query, create a secondary tableId using the query's itemId 
                    // and save the preferences under it. Any time the save query is used,
                    // the secondary table id is set to retrieve the settings.
                    if(queryObj.saveColumns) {
                        var table = self.tableView.tableViewer;
                        var isShared = query.scope == FiltersQueriesClient.publicScopeId;
                        table.saveTableSettingsToPreference(query.itemId, /*write to project scope*/isShared);
                    }
                    if (originalQueryObj) {
                        // Remove original query in cache if it is specified i.e. this is an edit.
                        // In the case that the edit is in the same folder we can't remove the folder
                        // if it has only one query. Otherwise the folder referenced by parentItem
                        // will be invalid even a new folder is created later on.
                        var sameLoc = (queryObj.scope == originalQueryObj.scope) && (queryObj.folder == originalQueryObj.folder);
                        self._deleteOrganizedQuery(originalQueryObj, sameLoc); // true to keep folder even if it is empty
                    }
                    // update query in cache
                    self._organizeQuery(savedQuery);
                    if (originalQueryObj && originalQueryObj.inMenu) {
                        // edited query is in the app menu, need to update the menu too
                        self.queriesConfigChanged(successCallback, [parentItem, queryItem, savedQuery, originalQueryObj]);
                    } else {
                        successCallback(parentItem, queryItem, savedQuery, originalQueryObj);
                    }
                }
            },
            error: function(e){
                if (errorCallback) {
                    errorCallback(e);
                } else {
                    console.error(e);
                }
            }
        },
        "success",
        "error");
        
        if (queryObj.scope == FiltersQueriesClient.autoSaveScopeId) {
            // auto-save query
            FiltersQueriesClient.autoSaveQueries(handler, queryParam);
        }
        else if (updateQuery) {
            // update existing query
            FiltersQueriesClient.updateQueries(handler, queryParam);
        } else {
            // save new query
            FiltersQueriesClient.saveQueries(handler, queryParam);
        }
    },

    deleteQuery: function(parentItem, queryItem, successCallback, errorCallback) {
        var confirmDialog = new com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog({
            message: Constants.getMessage("msgDeleteQuery", queryItem.query.title),
            primaryTitle: Constants.getMessage("labelConfirmation"),
            okCallback: dojo.hitch(this, "_deleteQuery", parentItem, queryItem, successCallback, errorCallback)
        });
        confirmDialog.open();
    },
    
    _deleteQuery: function(parentItem, queryItem, successCallback, errorCallback) {
        // make sure the query exists
        var queryObjParam = queryItem.query;
        var existingQuery = this.findQuery(queryObjParam);
        if (!existingQuery) {
            var error = new Error("invalid queryObjParam passed to delete"); // $NON-NLS$ dev usage error
            if (errorCallback) {
                errorCallback(error);
            } else {
                console.error(error);
            }
            return;
        }
        
        // prepare the query param to delete
        var queryParams = [{
            "artifactType": this.artifactTypeQueryKey,
            "itemId": existingQuery.query.itemId,
            "stateId": existingQuery.query.stateId
        }];

        var self = this;
        var handler = new ServiceResponseHandler({
            success: function(response){
                // delete query from cache
                self._deleteOrganizedQuery(queryObjParam);
                successCallback(parentItem, queryItem);
                // delete query is in the app menu, need to update the menu too
                if (queryObjParam.inMenu) {
                    self.queriesConfigChanged();
                }
            },
            error: function(e){
                if (errorCallback) {
                    errorCallback(e);
                } else {
                    console.error(e);
                }
            }
        },
        "success",
        "error");

        FiltersQueriesClient.deleteQueries(handler, queryParams);
    },

    togglePromotedQuery: function(parentItem, queryItem, successCallback, errorCallback) {
        // to promote to menu, check if the menu item group of the saved queries reached the max already which is not allowed to promote
        if (!queryItem.query.inMenu) {
            var isPublic = queryItem.query.scope == FiltersQueriesClient.publicScopeId;
            var objectedMenu = this._isPromotedQueriesMax(queryItem.query.type, isPublic);
            if (objectedMenu) {
                var menuGroupTitle = Constants.getMessage(isPublic ? "labelBrowseSharedQueriesTitle" : "labelBrowseMyQueriesTitle");
                var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog({
                    'message': Constants.getMessage("msgNoPromoteSavedQueryMax", menuGroupTitle, objectedMenu)
                });
                alertDialog.open();
                return;
            }
        }
        var confirmMessageKey = queryItem.query.inMenu ? "msgRemoveQueryFromMenu" : "msgAddQueryToMenu";
        var message = Constants.getMessage(confirmMessageKey, queryItem.query.title);
        if (queryItem.query.scope == FiltersQueriesClient.publicScopeId) {
            message += "<br>" + Constants.getMessage("msgChangeAffectProjectScope");
        }
        var confirmDialog = new com.ibm.asq.common.web.ui.internal.widgets.ConfirmationDialog({
            message: message,
            primaryTitle: Constants.getMessage("labelConfirmation"),
            okCallback: dojo.hitch(this, "_togglePromotedQuery", parentItem, queryItem, successCallback, errorCallback)
        });
        confirmDialog.open();
    },
    
    _isPromotedQueriesMax: function(queryType, isPublic) {
        var controller = dojo.getObject("com.ibm.rqm.web.homePageController");
        var homePageView = controller._view;

        var menuItemsEntries;
        var maxEntriesName = "";
        if (homePageView.actionBarIsHorizontal) {
            // get navBar menu item entries for horizontal menu bar
            menuItemsEntries = dojo.getObject("com.ibm.rqm.web.globalNavBarItems");
        } else {
            // get the menu item entries for vertical action bar
            menuItemsEntries = homePageView._actionBar.allEntries;
        }
        for (var i = 0; i < menuItemsEntries.length; i++) {
            var menuItemsEntry = homePageView.actionBarIsHorizontal ? menuItemsEntries[i].parentActionBarEntry : menuItemsEntries[i];
            if (menuItemsEntry.isPromotedSavedQueriesMax(queryType, isPublic)) {
                maxEntriesName += "<br> &nbsp; &nbsp; &nbsp; " + menuItemsEntry.title;
            }
        }
        return maxEntriesName;
    },
    
    _togglePromotedQuery: function(parentItem, queryItem, successCallback, errorCallback) {
        // make sure the query exists
        var queryObjParam = queryItem.query;
        var existingQuery = this.findQuery(queryObjParam);
        if (!existingQuery) {
            var error = new Error("invalid queryObjParam passed to promote"); // $NON-NLS$ dev usage error
            if (errorCallback) {
                errorCallback(error);
            } else {
                console.error(error);
            }
            return;
        }
        
        // toggle the inMenu setting
        var query = existingQuery.query;
        query.inMenu = queryItem.query.inMenu = !query.inMenu;
        
        // prepare params to update query
        var queryParams = [{
            "artifactType": this.artifactTypeQueryKey,
            "itemId": query.itemId,
            "stateId": query.stateId,
            "isMenu": query.inMenu ? "true" : "false",
            "scope": query.scope,
            "folder": query.folder,
            "name": query.title,
            "filters": [ query.filters ]
        }];

        var self = this;
        var handler = new ServiceResponseHandler({
            success: function(response){
                self.queriesConfigChanged(successCallback, [ parentItem, queryItem ]);
            },
            error: function(e){
                if (errorCallback) {
                    errorCallback(e);
                } else {
                    console.error(e);
                }
            }
        },
        "success",
        "error");

        FiltersQueriesClient.updateQueries(handler, queryParams);
    },

    queriesConfigChanged: function(/* optional */callback, /* optional */ callbackData) {
        // get the dropdown button on the app fly-out menu for the view queries action
        var inDropdown = this.artifactTypeQueryKey;
        // invalidate the cached inMenu queries
        FiltersQueriesClient.getAllMenuQueries(dojo.hitch(this, function() {
            this._refreshFilterQueryMenuItems(inDropdown);
            // check for nonDispatchable clone as well
            this._refreshFilterQueryMenuItems(inDropdown, true);
            if (callback) {
                if (callbackData) {
                    if (dojo.isArray(callbackData)) {
                        callback.apply(this, callbackData);
                    } else {
                        callback(callbackData);
                    }
                } else {
                    callback();
                }
            }
        }), inDropdown);
    },
    
    _refreshFilterQueryMenuItems: function(inDropdown, nonDispatchable) {
        var inDropdownKey = nonDispatchable ? "nonDispatchable-" + inDropdown : inDropdown;
        var dropdownEntry = dojo.getObject(inDropdownKey);
        if (dropdownEntry) {
            // regenerate the action entries within the dropdown button
            dropdownEntry.actionbarEntry.removeAllDropdownMenuItems(inDropdown);
            dropdownEntry.actionbarEntry._addFilterQueryMenuItem(dropdownEntry.title, dropdownEntry.method, dropdownEntry.actionArgs, inDropdown);
        }
    },
    
    getActionsInfo: function() {
        var actions = [
            {
                id: "cancel",
                title: Constants.getMessage("labelClearFilter"),
                iconURL: ImageCache._getIconUri("elcl16/clear.gif"),
                callback: dojo.hitch(this, "filterResetActionInvokedWrapper"),
                callbackData: null,
                isDefault: false
            }, {
                id: "run",
                title: Constants.getMessage("labelRun"),
                callback: dojo.hitch(this, "filterRunActionInvokedWrapper"),
                callbackData: null,
                isDefault: true,
                needFiltersValidation: true
            }
        ];
        if (this.saveQueriesSupport && this.tableView && this.tableView.showExplorer && !this.tableView.saveQueriesReadOnly) {
            actions.unshift({
                id: "save",
                title: Constants.getMessage("labelSave"),
                iconURL: ImageCache._getIconUri("elcl16/save.gif"),
                callback: dojo.hitch(this, "saveQuery"),
                callbackData: null,
                isDefault: false,
                needFiltersValidation: true
            });
        }
        return actions;
    },
    
    filterResetActionInvokedWrapper: function(actionName, filters) {
        // clear the phantom selected query, query that is not from the current user
        this.setPhantomSelectedQuery(null);
        // subclass might override the actual reset action
        this.filterResetActionInvoked(actionName, filters);
    },
    
    filterResetActionInvoked: function(actionName, filters) {
        // reset the filters.
        this.tableView.tableViewer.clearContentFilters();
        if(this.tableView.clearSubcategoryFilterSelection)
        	this.tableView.clearSubcategoryFilterSelection(filters);
        this.tableView.filteredArgs = {};
    },
    
    
    createCategoryFilterHierarchy: function(allCategoriesInArray, allCategoryTypesMap){
    	//We will format the flat category types/values into a hierarchy  type based structure
    	var len = allCategoriesInArray?allCategoriesInArray.length:0;
    	var typesToRemove = [];
    	for(var i=0;i< len; i++){
    		var categoryType = allCategoriesInArray[i];    	
    		if(categoryType.dependsOn){
    			var parentType = allCategoryTypesMap[categoryType.dependsOn];
    			if(parentType){
    				typesToRemove.push(categoryType.itemId);
    				if(!parentType.childTypeMap){
    					parentType.childTypeMap = {};
    				}
    				parentType.childTypeMap[categoryType.itemId] = categoryType;    			
    			}
    		}
    	}
    	len = typesToRemove.length;
    	for(var i=0;i<len;i++){
    		delete allCategoryTypesMap[typesToRemove[i]];
    	}
    	return allCategoryTypesMap;
    },
    
    formatCategoryFilterTypes: function(types, parent, allCategoryTypesMap){
    	//After we got the proper hierarchy now we will proceed to create the  hierarchy based on user selections in a parent-child category value relationship
    	for(var key in types){
    		if(types[key]){
    			var type = types[key];
    			if(type.childTypeMap){
    				this.formatCategoryFilterTypes(type.childTypeMap, type, allCategoryTypesMap);
    			}    			
    			if(parent){
    				var selectedCategories = type.selectedCategories;
    				var removeValues = [];
    				var len =  selectedCategories? selectedCategories.length:0;    				
    				for(var j=0;j<len;j++){    					
    					var mycategoryValue = selectedCategories[j];
    					var parentCategoryValues = this.findCategoryValuesFromType(parent, mycategoryValue.parentValueItemId);    					
    					if(parentCategoryValues == 0){
        					//no matching parent value found, this means that either we selected unassigned or somehow this value is not in hierarchy, so 
    						//we need to create a copy of this type in root and assign the values    						
    						if(!allCategoryTypesMap[type.itemId]){
    							allCategoryTypesMap[type.itemId] = {
    									itemId: type.itemId,
    									name: type.name,
    									selectedCategories: []    								    									
    							};
    						}
    						allCategoryTypesMap[type.itemId].selectedCategories.push(mycategoryValue);
    						removeValues.push(i);
    						//since this value has been restored in root, we need to delete it from the original array
    					} else {
    						for (var i=0;i<parentCategoryValues.length;i++){
    							var parentCategoryValue = parentCategoryValues[i];
    							if(!parentCategoryValue.childTypes){
    								parentCategoryValue.childTypes = [];    						
    							}
    							var childTypes = parentCategoryValue.childTypes;
    							if(!childTypes[type.itemId]){
    								childTypes[type.itemId] = {
    										selectedCategories: []
    								};
    							}    					
    							childTypes[type.itemId].selectedCategories.push(mycategoryValue);
    						}
    					}
    				}
    				len = removeValues.length;
    				for(var i=0;i<len;i++){
    					//just to delete values from the original array
    					selectedCategories.splice(removeValues[i]);
    				}
    			}
    		}
    	}
    },
    
    createCategoryJSONFromHierarchy: function(hierarchy){
    	//Create the  final structure for the json, here we will extract the ItemIds and arrange them in hierarchy mode 
    	var JSON = {};
    	for(var key in hierarchy){
    		var categoryType = hierarchy[key];
    		var values = categoryType.selectedCategories;    		    		    		
    		var len = values?values.length:0;
    		var valuesJSON = {};
    		for(var i=0;i<len;i++){
    			var value =  values[i];    			
    			var childTypes = value.childTypes;
    			valuesJSON[value.itemId] = {};
    			if(childTypes){    				
    				valuesJSON[value.itemId] = this.createCategoryJSONFromHierarchy(childTypes);	    				    			
    			}
    		}
    		JSON[key] = valuesJSON;
    	}
    	return JSON;
    },
    
    prepareCategoryArgs: function(allCategoriesInArray, allCategoryTypesMap){   
    	/*
    	 * We need to send a json indicating the structure of the hierarchy along with selected values, 
		  for example, suppose we have 3 levels,  the json will contain: 
		  
		  { ParentTypeUUID: { 
		  				Value1UUID: { }
		  				value2UUID: { 
		  							SubType1UUID: { 
		  				 								SubValue1UUID: {}
		  				 								SubValue2UUID: {
		  				 												SubsubTypeUUID: {
		  				 																subSubValue1UUID: {}
		  				 												}
		  				 							}
		  				 			}
		  				}
    	 */
    	var hierarchy = this.createCategoryFilterHierarchy(allCategoriesInArray, allCategoryTypesMap);
    	this.formatCategoryFilterTypes(hierarchy, null, allCategoryTypesMap);
    	return  this.createCategoryJSONFromHierarchy(hierarchy);    	
    },               
    
    findCategoryValuesFromType: function(type, valueItemId){
    	var values = [];
    	var len = type.selectedCategories?type.selectedCategories.length:0;
    	for(var i=0;i<len;i++){
    		var value = type.selectedCategories[i];
    		if(value.itemId == valueItemId){
    			values.push(value);
    		}
    	}
    	return values;
    },
    
    prepareFilterArgs: function(filters) {
        var allCategoryTypes = [];  
        var allCategoryTypesMap = null;
        var args = {};
        for (var nameKey in filters) {
            // iterate through the filters and skip those that are not specified
            if (filters[nameKey] != null) {
                if (nameKey.indexOf("categories") == 0) {
                	
                    // filter name has "categories" prefix
                    var values = filters[nameKey];
                    if (values && values.length > 0) {
                    	var valuesCopy = [];
                    	var len = values.length;
                    	for(var i=0;i<len;i++){
                    		var value = values[i];

                    		if(typeof value === 'object'){

                    			//create a clean copy of the value containing only the information we need,
                    			//we don't want to create dirty content inside
                    			valuesCopy.push({
                    				itemId:value.itemId,
                    				name: value.name,
                    				parentValueItemId: value.parentValueItemId,
                    				parentCatTypeItemId: value.parentCatTypeItemId,
                    				currentCatTypeItemId: value.currentCatTypeItemId
                    			});
                    		} else {
                    			//support for legacy filters
                    			valuesCopy.push({
                    				itemId: value
                    			});
                    		}
                    	}                    	
                        var filter = this.getFilterByName(this.filters, nameKey);
                        if (filter) {                        	
                        	var originalCatType = filter.params.categoryTypeObject;;
                            var categoryType = {
                            		itemId: originalCatType.itemId,
                            		name : originalCatType.name,
                            		dependsOn: originalCatType.dependsOn,
                            		selectedCategories: []
                            };
                            categoryType.selectedCategories = categoryType.selectedCategories.concat(valuesCopy);          
                            allCategoryTypes.push(categoryType);
                            if(!allCategoryTypesMap){
                            	allCategoryTypesMap = {};
                            }
                            allCategoryTypesMap[categoryType.itemId] = categoryType;
                        }
                    }
                }
				else if (nameKey == "id") {
					if(filters[nameKey].value){
	                    // re-map id to separate comparator and value
	                    args["idValue"] = filters[nameKey].value;
	                    args["idComparator"] = filters[nameKey].operator;
					}
					else{
						args[nameKey] = filters[nameKey];
					}
                }
                else if (nameKey == "weight") {
                    // re-map weight to separate comparator and value
                    args["weightValue"] = filters[nameKey].value;
                    args["weightComparator"] = filters[nameKey].operator;
                }
				else if (nameKey == "timeSpent") {
                    args['timeSpentComparator'] = filters[nameKey].operator;
                    args['timeSpentValue'] = filters[nameKey].value;
                }
				else if (nameKey == "estimate") {
                    args["estimateValue"] = filters[nameKey].value;
                    args["estimateComparator"] = filters[nameKey].operator;
                }
                else {
                    args[nameKey] = (filters[nameKey] == undefined) ? "" : filters[nameKey];
                    if (args[nameKey] == 'Any@@') 
                        args[nameKey] = '';
                }
            }
        }
       
        if (allCategoryTypes && allCategoryTypes.length > 0) {
            args["categories"] =  dojo.toJson(this.prepareCategoryArgs(allCategoryTypes, allCategoryTypesMap));
        }
        return args;
    },
    
    filterRunActionInvokedWrapper: function(actionName, filters, updateOnly) {
        // clear the phantom selected query
        this.setPhantomSelectedQuery(null);
        // subclass might override the actual run action
        this.filterRunActionInvoked(actionName, filters, updateOnly);
    },
    
    filterRunActionInvoked: function(actionName, filters, updateOnly){
        if (this.tableView) {
            // must have the view context to perform the query and update the table display
            var args = this.prepareFilterArgs(filters);
            // reset the paging for new query
            this.tableView.queryToken = null;
            var isUpdateOnly = (updateOnly != undefined) ? updateOnly : true;
            this.tableView.runQuery(args, isUpdateOnly);
        }
    },

    // common filter provider methods
    getTitle: function() {
        return Constants.getMessage("labelFilterTitle");    
    },
    
    getIsSectionOpenInitially: function() {
       return this.initOpenFilter; 
    },
    
    getFilterCount: function() {
        return this.filters.length;
    },

    getFilters: function() {
        return this.filters;
    },
        
    getFilterIdName: function(index) {
        var filter = this.filters[index];
        return filter.dbName || filter.name;        
    },

    getFilterName: function(index) {
        return this.filters[index].name;
    },
	
	getColumnKeyword: function(atColumnIndex)
	{
		var filterName = this.filters[atColumnIndex].name;
		
		if(filterName.indexOf("categories") != -1){
			return "isCategoryColumn";
		}else if(filterName.indexOf("customFilter") != -1){
			return "isCustomColumn";
		}else if(filterName.indexOf("reqCollLink") != -1){
		 	return "trace";
		}else if(filterName.indexOf("devPlanLink") != -1){
			return "trace";
		}else{
			return "";
		}
	},
    
    getFilterTitle: function(index) {
        return this.filters[index].title;
    },
	
	getFilterTooltip: function(index) {
        return this.filters[index].tooltip;
    },
    
    getFilterPromptTitle: function(index) {
        return this.filters[index].promptTitle;        
    },
    
    getFilterType: function(index) {
        return this.filters[index].type;
    },

    getFilterUniqueId: function(index) {
        return this.filters[index].uniqueId || null;  
    },
    
    getFilterDefaultValue: function(index) {
        return this.filters[index].defaultValue;
    },
    
    getFilterParams: function(index) {        
        return this.filters[index].params;
    },

    getCustomField: function(index) {
        return null;
    },
    
    // Phantom query is query that is not from the current user.  It is mainly for personal
    // saved query set in a dashboard viewlet that other users can not get to.
    //
    setPhantomSelectedQuery: function(queryArgs) {
        if (queryArgs) {
            // phantom query is not a real query
            delete this.selectedQuery;
            // expand the filter area to make sure the filters and message box is visible
            var viewer = this.tableView.tableViewer;
            viewer.toggleFiltersDisplay(true);
            
            this.phantomSelectedQuery = queryArgs;
            var messageKey = "msgSavePhantomQuery" + (queryArgs.sqTitle ? "WithName" : "");
            var message = Constants.getMessage(messageKey, queryArgs.sqTitle);
            this.showMessageBox(message);
        } else if (this.phantomSelectedQuery) {
            // setting phantom query to null is to clear it
            delete this.phantomSelectedQuery;
            this.clearMessageBox();
        }
        
        // notify the table view if it handle phantom query as well
        if (this.tableView && dojo.isFunction(this.tableView.setPhantomSelectedQuery)) {
            this.tableView.setPhantomSelectedQuery(queryArgs);
        }
    },
    
    invalidateSavedQuery: function() {
        if (!this.tableView.saveQueriesReadOnly && !this.isIgnoreValueChanged() && this.selectedQuery && !this.phantomSelectedQuery
        	&& (this.selectedQuery.scope!=FiltersQueriesClient.autoSaveScopeId)) {
            this.showMessageBox(Constants.getMessage("msgQueryAttrModified"));
        }
    },
    
    onFilterValueChanged: function(fieldWidgetRef, oldValue, newValue) {
        this.invalidateSavedQuery();
    },

    onPerformColumnSort: function(sortColumns) {
        this.invalidateSavedQuery();
    },
    
    onPerformGroupBy: function(colIndex) {
        this.invalidateSavedQuery();
    },
    
    onQuerySelected: function(query) {
        // a query is selected or cleared in the tableView
        this.selectedQuery = query;
        // clear the message box either a new query is selected or cleared
        this.clearMessageBox();
        // we have selected a real query, clear the phantom selected query
        this.setPhantomSelectedQuery(null);
    },

    getMessageBox: function() {
        this.messageBox = null; 
        if (this.saveQueriesSupport && this.tableView && this.tableView.showExplorer) {
            var messageBox = new jazz.ui.MessageBox({});
            dojo.style(messageBox.domNode, {
                "display" : "inline-block",
                "verticalAlign" : "middle",
                "lineHeight": "1"
            });
            // the new message box may be created from recreated table header, restore the previous message
            this.messageBox = messageBox;
            this.showMessageBox();
        }
        return this.messageBox;
    },
    
    showMessageBox: function(message) {
        if (message) {
            this.filterMessage = message;
        }
        if (this.filterMessage && this.messageBox && !this.messageBox._beingDestroyed && !this.messageBox._destroyed) {
            // the message box may be destroyed if the table header is recreated
            var messageArgs = {
                summary: this.filterMessage,
                type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
                closable: false
            };
            dojo.style(this.messageBox.domNode, "display", "inline-block");
            this.messageBox.showMessage(messageArgs);
        }
    },
    
    clearMessageBox: function() {
        this.filterMessage = null;
        if (this.messageBox && !this.messageBox._beingDestroyed && !this.messageBox._destroyed) {
            // need to explicitly set display to none since we override the message box display to inline-block
            dojo.style(this.messageBox.domNode, "display", "none");
            this.messageBox.hideMessage();
        }
    }

});
})();
